/**
 * Copyright (C) 1997-2000 Dallas Semiconductor Corporation.
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 */
/* TAC 06/28/99 */
/*
 $Workfile: BuildA51.java $
 $Revision: 26 $
 $Date: 11/16/00 9:56a $
 $Author: Chenot $
 $Modtime: 11/16/00 9:56a $
 $Log: /TINI/390 Assembler/source/BuildA51/BuildA51.java $
* 
* 26    11/16/00 9:56a Chenot
* 
* 25    5/20/00 9:13p Chenot
*
* 24    5/20/00 6:05p Chenot
*
* 23    5/16/00 8:09a Chenot
*
* 22    2/17/00 10:10a Tomc
*
* 21    2/16/00 9:05a Tomc
*
* 20    1/06/00 7:23a Tomc
*
* 19    11/17/99 5:57a Tomc
*
* 18    10/15/99 2:31p Tomc
*
* 16    10/01/99 6:38a Tomc
*
* 15    10/01/99 6:31a Tomc
*
* 14    9/15/99 6:18a Tomc
*
* 13    7/14/99 6:47a Tomc
*
* 12    7/13/99 1:15p Tomc
*
* 9     7/12/99 11:51a Tomc
*
* 8     7/09/99 11:04a Tomc
*
* 7     7/09/99 9:57a Tomc
*/

import java.io.*;
import java.util.*;

/**
  */
public class BuildA51
{
  private static final String versionString = "BuildA51, Beta Version 0.111";
/**
  */
  private static String buildString = "$Revision: 26 $";
  private static String dateString = "$Date: 11/16/00 9:56a $";
/**
  */
  private String makeFileName;
/**
  */
  private static String binaryFolder;
/**
  */
  private static String dependancyFileName;
/**
  */
  private static String outputFileName = "glob.a51";
/**
  */
  private static boolean buildAll;
/**
  */
  private static boolean allowDirectOverlap;
/**
  */
  private static boolean dashL;
/**
  */
  private static boolean dashD;
  /**
  */
  private static boolean quietMode;
/**
  */
  private static String outputFormat;
/**
  */
  private static boolean verboseMode;
/**
  */
  private static boolean statMode;
/**
  */
  private static int outputBufferSize = 16384;
/** Constructor
  */
  public BuildA51(String makeFileName)
  {
    this.makeFileName = makeFileName;

    if(binaryFolder == null)
      binaryFolder = "";
  }
/** Constructor
  */
  public BuildA51()
  {
    this("makefile");
  }
/**
  */
  public static void setBinaryFolder(String newBinaryFolder)
  {
    if(newBinaryFolder == null)
      binaryFolder = "";
    else if (!newBinaryFolder.endsWith(File.separator))
      binaryFolder = newBinaryFolder + File.separator;
    else
      binaryFolder = newBinaryFolder;
  }
/**
  */
  public static void setOutputFile(String newOutputFileName)
  {
    if(newOutputFileName == null)
      return;

    outputFileName = newOutputFileName;
  }
/**
 * Runs the given command in a separate environment, and displays
 * any output from that process in the message window.
 *
 * @param command  the statement that would normally be typed at the command line
 *
 * @result the success of the command
 */
  public static boolean runCommand(String[] command)
  {
    String  s;
    Process p = null;

    try
    {
      //Do the command.
      p = Runtime.getRuntime().exec(command);

      //Capture the output streams of the process.
      BufferedReader i    = new BufferedReader(new InputStreamReader(p.getInputStream()));
      BufferedReader err  = new BufferedReader(new InputStreamReader(p.getErrorStream()));

      while ((s = err.readLine()) != null)
      {
        System.out.println(s);
      }

      while ((s = i.readLine()) != null)
      {
        System.out.println(s);
      }

      try
      {
        p.waitFor();
      }
      catch(InterruptedException ie)
      {
        System.out.println("runCommand:  exec thread interrupted.");
      }

      //The command completed successfully.
      if(p.exitValue() == 0)
        return true;
      else
        return false;
    }
    catch(IOException e)
    {
      e.printStackTrace();
      return false;
    }
  }
/**
  */
  String getFileName(String stringToParse)
  {
    return getFileName(stringToParse,0);
  }
/**
  */
  String getFileName(String stringToParse, int index)
  {
    String  currentFileName = new String(stringToParse).replace('\t',' ');
    int     startIndex  = 0;

    currentFileName = currentFileName.trim();
    int subStringLength = currentFileName.indexOf("#");

    if(subStringLength != -1)
      currentFileName = currentFileName.substring(0, subStringLength).trim();

    for(int i = 1; i <= index; i++)
    {
      startIndex = currentFileName.indexOf(' ');

      if(startIndex == -1)
        return null;

      currentFileName = currentFileName.substring(startIndex).trim();
    }

    subStringLength = currentFileName.indexOf(' ');

    if(subStringLength != -1)
      currentFileName = currentFileName.substring(0, subStringLength).trim();

    currentFileName = currentFileName.trim();
    return (currentFileName.length() > 0)? currentFileName: null;
  }
/**
  */
  void commitIniFile(IniFile iniFile)
  {
    String [] fileNames = iniFile.getGroups();
    long    newValue = 0;
    long    oldValue = 0;
    String  groupName = null;
    String  longValue;

    for(int i = 0; i < fileNames.length; i++)
    {
      try
      {
        groupName = fileNames[i]/*.toUpperCase()*/;
        longValue = iniFile.getValue(groupName, "LastModified");
        newValue  = (new File(fileNames[i])).lastModified();

        if(longValue == null)
          iniFile.setValue(fileNames[i]/*.toUpperCase()*/, "LastModified", new String(Long.toString(newValue)));
        else
        {
          oldValue  = Long.valueOf(longValue).longValue();

          if(newValue > oldValue)
            iniFile.setValue(fileNames[i]/*.toUpperCase()*/, "LastModified", new String(Long.toString(newValue)));
        }
      }
      catch(NullPointerException e)
      {
        iniFile.setValue(fileNames[i]/*.toUpperCase()*/, "LastModified", new String(Long.toString(newValue)));
      }
    }

    try
    {
      iniFile.writeToDisk();
    }
    catch(IniFileException e)
    {
      //newIniFile = true;
    }
  }
/**
  */
  public class MakeList
  {
    public  String  fileName;
    public  boolean isModified;

/**
  */
    public MakeList()
    {
      this(null, false);
    }
/**
  */
    public MakeList(String fileName)
    {
      this(fileName, false);
    }
/**
  */
    public MakeList(String fileName, boolean isModified)
    {
      this.isModified = isModified;
      this.fileName   = fileName;
    }
  }
/** Extracts file names from the file makeFileName and executes macro and a390 using these file names.
  *
  */
  void build()
  {
    MakeList[]        makeList  = new MakeList[100];
    String            currentPath;
    String            nextDependancyName;
    File              currentFile;
    int               lineCount = 0;
    //FileOutputStream  fileOutputStream;
    BufferedOutputStream  fileOutputStream;

    currentPath = (new File(".")).getAbsolutePath() + File.separatorChar;

    if(dependancyFileName == null)
      dependancyFileName = currentPath + "BuildA51.ini";

    IniFile iniFile = new IniFile (dependancyFileName, versionString);

    try
    {
      iniFile.readFromDisk();
    }
    catch(IniFileException e)
    {
      //newIniFile = true;
    }

    // extract the file names
    try
    {
      BufferedReader  bufferedReader = new BufferedReader(new FileReader(makeFileName));
      String          currentFileName,
                      currentLine;

      while((currentLine = bufferedReader.readLine()) != null)
      {
        currentFileName = getFileName(currentLine);

        int   nextDependancy;
        long  currentValue,
              fileValue;

        if(verboseMode)
          if(currentFileName != null)
            System.out.println("Dependancies for "+currentFileName+":");

        if(currentFileName != null)
        {
          makeList[lineCount] = new MakeList(currentFileName);
          nextDependancy = 1;

          while((nextDependancyName = getFileName(currentLine, nextDependancy++)) != null)
          {
            if(verboseMode)
              System.out.print(nextDependancyName);

            String currentValueString;

            if((currentValueString = iniFile.getValue(nextDependancyName/*.toUpperCase()*/, "LastModified")) == null)
            {
              if(verboseMode)
                System.out.println(" (new)");

              makeList[lineCount].isModified = true;
              currentValue    = new File(nextDependancyName).lastModified();
              iniFile.addPair(nextDependancyName/*.toUpperCase()*/, "LastModified", new String(Long.toString(currentValue)));
              //break;
            }
            else
            {
              try
              {
                currentValue = Long.valueOf(currentValueString).longValue();
              }
              catch(Exception e)
              {
                System.out.println(e);
                break;
              }

              File depFile = new File(nextDependancyName);

              if(!depFile.exists())
              {
                if(verboseMode)
                  System.out.println();

                commitIniFile(iniFile);
                System.out.println("Cannot find " + nextDependancyName);
                return;
              }

              if((fileValue = depFile.lastModified()) > currentValue)
              {
                makeList[lineCount].isModified = true;

                if(verboseMode)
                  System.out.println(" (modified)");

                //break;
              }
              else
              {
                if(verboseMode)
                  System.out.println(" (current)");
              }
            }
          }

          if(currentFileName.length() > 0)
            makeList[lineCount++].fileName = currentFileName;
        }
      }
    }
    catch(FileNotFoundException e)
    {
      System.out.println("Could not find " + makeFileName);
      return;
    }
    catch(IOException e)
    {
      System.out.println(e);
      return;
    }

    // create the glob file
    try
    {
      fileOutputStream = new BufferedOutputStream(new FileOutputStream(currentPath + outputFileName), Math.min(outputBufferSize,16384));
          //BufferedInputStream fileInputStream = new BufferedInputStream(new FileInputStream(mppFile), (int)Math.min(mppFile.length(),Integer.MAX_VALUE));
    }
    catch(IOException e)
    {
      System.out.println(e);
      return;
    }

    // run the preprocessor
    for (int i = 0; i < lineCount; i++)
    {
      currentFile = new File(makeList[i].fileName);

      if(currentFile.isDirectory())
      {
        continue;
      }
      else if (makeList[i].fileName.indexOf(".a51") > -1)
      {
        int numParams = 2 + ((i < (lineCount-1))? 1: 0) + (quietMode? 1: 0);
        
        String[]  commandLine = new String[numParams];//i < (lineCount-1)? 4: 3];//{"","",""};
        int       nextCommand = 0;

        commandLine[nextCommand++] = binaryFolder + "macro";

        // use quiet mode
        if(quietMode)
          commandLine[nextCommand++] = "-q";

        if(i < (lineCount-1))
          commandLine[nextCommand++] = "-e-";

        commandLine[nextCommand] = makeList[i].fileName;

        // concatenate the output files
        try
        {
          String mppFileName  = makeList[i].fileName.substring(0, makeList[i].fileName.indexOf(".a51")) + ".mpp";
          File   mppFile = new File(currentPath + mppFileName);

          if(makeList[i].isModified || buildAll || !mppFile.exists() || (currentFile.lastModified() > mppFile.lastModified()))
          {
            if(!runCommand(commandLine))
            {
              System.out.println("Errors detected! Build aborted!");
              fileOutputStream.close();
              commitIniFile(iniFile);
              //return;
              System.exit(1);
            }
          }

          BufferedInputStream fileInputStream = new BufferedInputStream(new FileInputStream(mppFile), 
                                                                        (int)Math.min(mppFile.length(),Integer.MAX_VALUE));
          int             fileLength      = (int)mppFile.length();
          int             blockSize       = fileInputStream.available();
          int             bytesRead       = 0;
          byte[]          fileBuffer      = new byte[fileLength];
          byte[]          tempBuffer      = new byte[blockSize];

          while(bytesRead < fileLength)
          {
            fileInputStream.read(tempBuffer);
            System.arraycopy(tempBuffer,0,
                             fileBuffer,(int)bytesRead,
                             tempBuffer.length);
            bytesRead += blockSize;
          }

          fileInputStream.close();
          fileOutputStream.write(fileBuffer);
        }
        catch(FileNotFoundException e)
        {
          System.out.println(e);
        }
        catch(IOException e)
        {
          System.out.println(e);
        }
      }
    }

    commitIniFile(iniFile);

    // run the assembler on the glob file
    try
    {
      fileOutputStream.close();

      int numParams = 2;

      if(statMode)
        numParams++;

      if(quietMode)
        numParams++;

      if(dashL)
        numParams++;

      if(dashD)
        numParams++;

      if(allowDirectOverlap)
        numParams++;

      if(outputFormat != null)
        numParams++;

      String[] commandLine = new String[numParams];

      int nextCommand = 0;
      commandLine[nextCommand++] = binaryFolder + "a390";

      if(statMode)
        commandLine[nextCommand++] = "-s";
      
      if(quietMode)
        commandLine[nextCommand++] = "-q";

      if(dashL)
        commandLine[nextCommand++] = "-l";

      if(dashD)
        commandLine[nextCommand++] = "-d";

      if(allowDirectOverlap)
        commandLine[nextCommand++] = "-o";

      if(outputFormat != null)
        commandLine[nextCommand++] = "-F"+outputFormat;

      commandLine[nextCommand] = outputFileName;

      System.out.println("\n\n\n*** END OF MACRO FILES ***");
      System.out.println("*** PROCESSING \"" + outputFileName + "\" ***");

      if(!runCommand(commandLine))
      {
        System.out.println("Errors detected, Build aborted!");
        fileOutputStream.close();
        //return;
        System.exit(1);
      }

      fileOutputStream.close();
    }
    catch(IOException e)
    {
      System.out.println(e);
    }
  }
/** Displays a usage statement.
  */
  public static void  usage()
  {
    System.err.println("Usage:");
    System.err.println("\tjava BuildA51\t-f <inputFileList file> [-F <format>]\n\t\t\t[-b <binary file directory>]\n\t\t\t[-o <output file name>] [-B <outputBufferSize>]\n\t\t\t[-v] [-s] [-?] [-all]");
    System.err.println("\t-v \tverbose output\n\t-s \tprint statistics\n\t-all \trebuild all files");
    System.err.println("\t-F \toutput format i.e. hex, tlib (default), tbin\n\t-? \tprint usage\n");
  }
/**
  */
  public static void main(String[] argv)
  {
    Date  before = new Date();
    String  makeFileName  = null;
    GetOpt  myopts        = new GetOpt(argv, "f:F:b:B:o:Oqsvald?"); // the l is a hack for the dashAll option
    int     opt;
    int     beginIndex;
    int     endIndex;

    beginIndex  = buildString.indexOf(':') + 2;
    endIndex    = buildString.lastIndexOf('$') - 1;
    buildString = buildString.substring(beginIndex, endIndex);
    buildString = "0000".substring(buildString.length()) + buildString;
    beginIndex  = dateString.indexOf(':') + 1;
    endIndex    = dateString.lastIndexOf('$') - 1;
    dateString  = dateString.substring(beginIndex, endIndex);

    String banner = new String(" Dallas Semiconductor " + versionString + "." + buildString + " ");

    System.err.print("");

    for(int i = banner.length() - 2; i > 0; i--)
      System.err.print("");

    System.err.println("");

    System.err.println(banner);

    System.err.print("");

    for(int i = banner.length() - 2; i > 0; i--)
      System.err.print("");

    System.err.println("");
    System.err.println("Last build " + dateString + "\n");

    while ((opt = myopts.getopt()) != myopts.optEOF)
    {
      switch (opt)
      {
        case 'f':
          if((makeFileName = myopts.optArgGet()) == null)
          {
            BuildA51.usage();
            return;
          }

          System.out.println("Using input file \"" + makeFileName + "\"");
        break;

        case 'F':
          if((outputFormat = myopts.optArgGet()) == null)
          {
            BuildA51.usage();
            return;
          }

          System.out.println("Using output format \"" + outputFormat + "\"");
        break;

        case 'b':
          BuildA51.setBinaryFolder(myopts.optArgGet());
          System.out.println("Using binary files in \""+myopts.optArgGet() + "\"");
        break;

        case 'B':
          outputBufferSize  = (new Integer(myopts.optArgGet())).intValue();
          System.out.println("Using output buffer size "+outputBufferSize);
        break;

        case 'd':
          dashD = true;
          System.out.println("Allowing non-relocatable syntax");
        break;

        case 'o':
          BuildA51.setOutputFile(myopts.optArgGet());
          System.out.println("Using output file \""+myopts.optArgGet() + "\"");
        break;

        case 'v':
          verboseMode = true;
          System.out.println("Verbose mode on");
        break;

        case 's':
          statMode = true;
          System.out.println("Statistics mode on");
        break;

        case 'O':
          allowDirectOverlap = true;
          System.out.println("Overlapping directs allowed");
        break;

        case 'q':
          quietMode = true;
          System.out.println("Quiet mode on");
        break;

        case 'l':
          //if(argv[myopts.optIndexGet()].equals("-l"))
          {
            dashL = true;
            System.out.println("List file generation on");
          }
        break;

        case '?':
          BuildA51.usage();
        return;

        default:
          // hack for the dashAll option
          try
          {
            if(argv[myopts.optIndexGet()].equals("-all"))
            {
              BuildA51.buildAll = true;
              System.out.println("Rebuild all on");
              myopts.getopt();
              myopts.getopt();
            }
            else
            {
              System.out.println("Invalid argument \"" + argv[myopts.optIndexGet()]);
              BuildA51.usage();
              return;
            }
          }
          catch(ArrayIndexOutOfBoundsException e)
          {}
        break;
      }
    }

    if(makeFileName == null)
      makeFileName = new String("Makefile");

    (new BuildA51(makeFileName)).build();

    Date after = new Date();
    long difference = after.getTime()-before.getTime();
    System.out.println();
    System.out.println(after.toString());
    System.out.println("Total Runtime: " +  (double)difference/1000 + " seconds");
  }
}
