
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

import java.io.*;
import java.util.*;
import java.util.zip.*;


/*
 *  BuildDependency KLA
 *  @send complaints to bucket-bin@big-harddrive-in-the-sky.org
 *  @send glowing praise to tini@dalsemi.com with subject line "BuildDependency ROCKS!"
 *
 *  History:
 *  (as BuildOneWireProgram)
 *  09-15-2000 (0.00) First version, used TINI overlay to help
 *             transition from TINI1.01 to use OneWireAPI 0.00.
 *  11-14-2000 (0.01) Updated for TINI 1.02, which has the new OneWireAPI.
 *             Changed to pass unknown arguments to TINIConvertor,
 *             so that all the power of TINIConvertor is accessible.
 *  11-30-2000 (0.02) Updated to allow containers to be loaded from a JAR file.
 *  12-06-2000 (0.03) Fixed container 09 not having Scratchpad in its dependency list.
 *             Also fixed problem where non-jarred dependencies would not be picked up.
 *  (becomes BuildDependency between releases of TINI 1.02p1 ans 1.02p2)
 *  12-13-2000 (1.00) Several updates.  Becomes BuildDependency, capable of handling
 *             any build where a set of class files are needed (are dependencies) for
 *             an application.
 *             1. A Dependency group (a set of files that are built into a TINI application)
 *                can be named anything.  The complete class name of all dependency classes
 *                must be specified.
 *             2. Pseudo-variables can be used.  A dependency ROOT can be declared as
 *                "com.dalsemi.onewire.container".  Then, other dependencies can contain
 *                text like "%ROOT%.OneWireContainer18", which will resolve.
 *             3. Multiple jar files and paths can be searched for class files.  The current
 *                directory is always searched last.  The "-debug" option will tell you where files
 *                are being built from.
 *  12-20-2000 (1.01) Changed all StringTokenizers to recognize a comma as a legitimate separator in
 *             addition to a semicolon.  There was some worry on the TINI list that ';' might not
 *             work so happy on Solaris.
 *  01-23-2001 (1.02) Addition of OneWireContainer33 to defaults, to match the soon-to-be-released new
 *             owapi_dep.txt dependency file.  Also, as a side note, hopefully the class-cleanup
 *             problem everyone is complaining about will go away too.  It wasn't even my fault!
 *             Weasel-boy had put System.exit(0)'s all over TINIConvertor!
 *  01-24-2001 (1.03) Added a feature that allows multiple dependency files to be specified by sending
 *             a semi-colon OR comma separated list of files to the -x option.
 *  03-13-2001 (1.04) Fixed a bug where the default onewire mappings were never used.
 *  06-21-2001 (1.05) Allowed problems reported by Exceptions to show an abnormal exit code, so that
 *             applications using BuildDependency can tell when failures occur.
 *  11-19-2001 (1.06) Implemented some DM suggestions.  Also added support for adding a native library.  In the
 *             dependency file, use parenthesis to denote that a name is the name of a native library...
 *                # sample dependency line
 *                CAN=(canlib.tlib);com.dalsemi.system.CAN
 *  11-27-2001 (1.07) Skip over lines in the dependency file that start with '#'...allows for comments.
 *             Really at this point any line without an '=' is a comment, but this will allow me to change that
 *             later. Also added some defaults for TINI's move to modules.
 *  11-28-2001 (1.08) Added Jesse's long-ass PPP dependency
 */

public class BuildDependency
{
    static String DATE = "November 30, 2001";
    static String VERSION = "1.08";

    static Vector filesToBeDeleted = new Vector();
    static byte[] buffer = new byte[2048];
    static boolean DEBUG = false;
    static Vector pathNames = new Vector();
   /**
    * Method main
    *
    *
    * @param args
    *
    * @throws IOException
    *
    */
   public static void main (String[] args)
      throws Exception
   {
      if ((args == null) || (args.length == 0))
      {
         printUsageString();
         return;
      }
      boolean SUCCESS = false;
      Vector containers = new Vector();
      Vector natives = new Vector();
      Vector originals = new Vector();

      String dep_path   = "";


      boolean depListOnly = false;
      String depTarget = null;

      //first let's see if TINIConvertor is in our classpath
      try
      {
         Class c = Class.forName("TINIConvertor");
      }
      catch (Exception e)
      {
         System.out.println("TINIConvertor is not in your classpath!");
         System.out.println("***************************************\r\n");
         System.out.println("Make sure the file tini.jar is in your classpath.");
         System.out.println("You can do this from the command line by calling");
         System.out.println("    java -classpath PATH_TO_TINI.JAR\\tini.jar;%classpath% BuildDependency [OPTIONS]");
         System.out.println("\r\nIt may also be useful to include '.' in your classpath:");
         System.out.println("    java -classpath .;PATH_TO_TINI.JAR\\tini.jar;%classpath% ...");
         throw new RuntimeException("TINIConvertor is not in your classpath!");
      }

      Vector dependencyNames = new Vector(5);
      int totalArgs = args.length;

      for (int i = 0; i < args.length; i++)
      {
         String thisarg = args [i];

         if (thisarg.equals("-debug"))
         {
            DEBUG = true;
            args[i] = null;
            totalArgs--;
         }
         else if (thisarg.startsWith("-dep"))
         {
            if (thisarg.equals("-dep"))
            {
                depListOnly = true;
            }
            else
            {
                depListOnly = true;
                depTarget = thisarg.substring(4);
            }
         }
         else if (thisarg.equals("-p"))
         {

            //then the next string is the desired path to the containers
            i++;

            StringTokenizer st = new StringTokenizer(args [i], ";,");

            while (st.hasMoreTokens())
            {
                String token = st.nextToken();
                try
                {
                    File f = new File(token);
                    if (!f.exists())
                    {
                        System.out.println("Error: Cannot find the File/Directory "+token+", cannot use it to find dependency classes.");
                    }
                    else
                    {
                        if (f.isDirectory())
                        {
                            /*System.out.println("abs path is "+f.getAbsolutePath());
                            System.out.println("can path is "+f.getCanonicalPath());
                            System.out.println("path is "+f.getPath());*/
                            token = f.getAbsolutePath() + File.separator;
                        }
                        pathNames.addElement(token);
                    }
                }
                catch(Exception e)
                {
                    System.out.println("Unexpected error trying to validate path dependency classes: " +e.toString());
                    throw new RuntimeException("Unexpected error trying to validate path dependency classes: " +e.toString());
                }
            }
            args[i-1] = null;
            args[i] = null;
            totalArgs -= 2;
         }
         else if (thisarg.equals("-x"))
         {
            //then the next string is the desired path to the TINI dependency file
            i++;

            dep_path = args [i];

            args[i-1] = null;
            args[i]   = null;
            totalArgs -= 2;
         }
         else if (thisarg.equals("-add"))
         {
            i++;
            StringTokenizer st = new StringTokenizer(args[i],";,");
            while (st.hasMoreTokens())
            {
                dependencyNames.addElement(st.nextToken().trim());
            }
            args[i]   = null;
            args[i-1] = null;
            totalArgs -= 2;
         }
         //else just leave it and let TINIConvertor croak on it
      }

      initializeDependencyFile(dep_path);

      if (depListOnly)
      {
         if (depTarget==null)  //then print them all
         {
             Enumeration enum = dependency.keys();
             while (enum.hasMoreElements())
             {
                 String key = (String)enum.nextElement();
                 System.out.println("Dependency list for "+key+":");
                 String[] dep = getDependencies(key);
                 if (dep.length==0)
                 {
                     System.out.println("There are no dependencies for "+key+".");
                 }
                 else
                 {
                     for (int k=0;k<dep.length;k++)
                     {
                         System.out.println(k + " : "+dep[k]);
                     }
                 }
                 System.out.println("---------------------------------------");
             }
             return;
         }
         else  //then only print container %depTarget%'s depdendency list
         {
             System.out.println("Dependency list for dependency "+depTarget+":");
             String[] dep = getDependencies(depTarget);
             if (dep.length==0)
             {
                 if (dependency.get(depTarget)==null)
                 {
                     System.out.println("There is no entry for a dependency "+depTarget+".");
                     //don't toss an exception here, this is just for information
                     return;
                 }
                 else
                 {
                     System.out.println("There are no dependencies for "+depTarget+".");
                     //don't toss an exception here, this is just for information
                     return;
                 }
             }
             else
             {
                 for (int k=0;k<dep.length;k++)
                 {
                     System.out.println(k + " : "+dep[k]);
                 }
                 return;
             }
         }
      }


      try
      {
            for (int i=0;i<dependencyNames.size();i++)
            {
                //now add the dependencies here...
                String[] dep = getDependencies((String)dependencyNames.elementAt(i));

                for (int x = 0; x < dep.length; x++)
                {
                    if (DEBUG) System.out.println("Dep " + x + " : " + dep [x]);

                    String element = dep [x];

                    if (element.startsWith("(") && element.endsWith(")"))
                    {
                        //then it represents a native library
                        element = element.substring(1, element.length()-1);
                        if (originals.indexOf(element) == -1)
                        {
                            originals.addElement(element);
                            element = fileCheck(element);
                            natives.addElement(element);
                        }
                    }
                    else
                    {
                        element = element.replace('.', '/');
                        element = element + ".class";
                        if (originals.indexOf(element) == -1)
                        {
                            originals.addElement(element);
                            element = fileCheck(element);
                            containers.addElement(element);
                        }
                    }
                    //String original_element = element;

                }
            }

            String[] arguments = new String[2 * containers.size() + totalArgs + 2 * natives.size()];
            int j=0;

            for (int i = 0; i < containers.size(); i++)
            {
                arguments [j++] = "-f";
                arguments [j++] = (String)containers.elementAt(i);
                if (DEBUG)
                    System.out.println("-f arg: "+(String)containers.elementAt(i));
            }
            for (int i = 0; i < natives.size(); i++)
            {
                arguments [j++] = "-n";
                arguments [j++] = (String)natives.elementAt(i);
                if (DEBUG)
                    System.out.println("-n arg: "+(String)natives.elementAt(i));
            }
            for (int i=0;i<args.length;i++)
            {
                if (args[i]!=null)
                    arguments [j++] = args[i];
            }

            PrintStream out = System.out;
            try
            {

                PrintStream os = new PrintStream(new ByteArrayOutputStream());
                if (!DEBUG) System.setOut(os);

                TINIConvertor.main(arguments);

                System.setOut(out);
                SUCCESS = true;
                out.println("\r\n\r\nBuild successful");
            }
            catch (Throwable e)
            {
                System.setOut(out);
                out.println("\r\n\r\nCould not create the application: "
                                    + e.toString());
                e.printStackTrace();
            }
      }
      catch(Exception uhoh)
      {
           //need to clean up, which we'll do just below...so let this drain
           System.out.println("\r\n\r\nCould not create the application: "+uhoh.toString());
           System.out.println("Cleaning up...");
      }

      //clean up - delete any temporary files
      while (filesToBeDeleted.size()>0)
      {
          String curr_filename = (String)filesToBeDeleted.elementAt(0);
          filesToBeDeleted.removeElementAt(0);
          try
          {
              File death_file = new File(curr_filename);
              death_file.delete();
          }
          catch(Exception e)
          {
              System.out.println("Could not delete the file "+curr_filename);
              //drain it...let's try to delete the next file
          }
      }
      System.out.println("BuildDependency clean-up complete.");
      if (!SUCCESS)
      {
          System.out.println("The program did not successfully terminate.");
          System.out.println("Try running with the '-debug' option to determine the cause.");
          throw new RuntimeException("BuildDependency did not successfully complete");
      }
   }

   private static Hashtable dependency   = null;
   private static Vector dep_filename_list = new Vector();

   private static void initializeDependencyFile (String filename)
   {
      StringTokenizer st = new StringTokenizer(filename, ",;");
      Hashtable temp = new Hashtable();

      if (!st.hasMoreTokens())
      {
          addDependencyFile(null,temp);
      }

      while (st.hasMoreTokens())
      {
          String nextFileName = st.nextToken();
          if (dep_filename_list.indexOf(nextFileName)==-1)
          {
              addDependencyFile(nextFileName, temp);
              dep_filename_list.addElement(nextFileName);
          }
      }

      dependency = temp;
   }

   private static void addDependencyFile(String dep_filename, Hashtable temp)
   {

      boolean fileExists = false;
      try
      {
         File test_file = new File(dep_filename);
         if (test_file.exists())
             fileExists = true;
      }
      catch(Exception e)
      {
         //assume file doesn't exist...
      }


      if (fileExists)
      {
        try
        {
            FileInputStream fin  = new FileInputStream(dep_filename);
            BufferedReader  in   = new BufferedReader(new InputStreamReader(fin));
            String          line = in.readLine();

            while (line != null)
            {
                if (line.length() > 1)
                {

                    //line should be KEY=PROPERTY
                    int eq_index = line.indexOf('=');

                    if ((eq_index!=-1) && (line.charAt(0)!='#'))
                    {
                        String dep_name = line.substring(0, eq_index).trim();

                        if (line.length() > eq_index)
                            line = line.substring(eq_index+1);
                        else
                            line = "";

                        temp.put(dep_name, line);
                    }
                    //else there are no dependencies for this one
                }
                line = in.readLine();
            }
        }
        catch (Exception e)
        {
            System.out.println("Failed to find dependencies: " + e.toString());
            return;
        }

        dependency = temp;
        return;
      }
      //else...
      System.out.println("Could not find dependency file, using defaults");
      temp.put("CONT_ROOT",          "com.dalsemi.onewire.container");
      temp.put("UTILS_ROOT",         "com.dalsemi.onewire.utils");
      temp.put("ADAPTER_ROOT",       "com.dalsemi.onewire.adapter");
      temp.put("OneWireContainer01", "%CONT_ROOT%.OneWireContainer01");
      temp.put("OneWireContainer02", "%CONT_ROOT%.OneWireContainer02");
      temp.put("OneWireContainer04", "%CONT_ROOT%.OneWireContainer04;%UTILS_ROOT%.Bit;%CONT_ROOT%.ClockContainer;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer05", "%CONT_ROOT%.OneWireContainer05;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.SwitchContainer");
      temp.put("OneWireContainer06", "%CONT_ROOT%.OneWireContainer06;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer08", "%CONT_ROOT%.OneWireContainer08;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer09", "%CONT_ROOT%.OneWireContainer09;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankEPROM;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer0A", "%CONT_ROOT%.OneWireContainer0A;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer0B", "%CONT_ROOT%.OneWireContainer0B;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankEPROM;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank");
      temp.put("OneWireContainer0C", "%CONT_ROOT%.OneWireContainer0C;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer0F", "%CONT_ROOT%.OneWireContainer0F;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankEPROM;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank");
      temp.put("OneWireContainer10", "%CONT_ROOT%.OneWireContainer10;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.TemperatureContainer");
      temp.put("OneWireContainer12", "%CONT_ROOT%.OneWireContainer12;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankEPROM;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.SwitchContainer");
      temp.put("OneWireContainer13", "%CONT_ROOT%.OneWireContainer13;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankEPROM;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank");
      temp.put("OneWireContainer14", "%CONT_ROOT%.OneWireContainer14;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankAppReg;%CONT_ROOT%.MemoryBankEE;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.PagedMemoryBank");
      temp.put("OneWireContainer16", "%CONT_ROOT%.OneWireContainer16;%CONT_ROOT%.CommandAPDU;%CONT_ROOT%.JibComm$BlockDataFragmenter;%CONT_ROOT%.JibComm;%CONT_ROOT%.ResponseAPDU");
      temp.put("OneWireContainer18", "%CONT_ROOT%.OneWireContainer18;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankNVCRC;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.MemoryBankScratchCRC;%CONT_ROOT%.MemoryBankScratchEx;%CONT_ROOT%.MemoryBankScratchSHA;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad;%CONT_ROOT%.SHAiButton");
      temp.put("OneWireContainer1A", "%CONT_ROOT%.OneWireContainer1A;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankNVCRC;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.MemoryBankScratchEx;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer1D", "%CONT_ROOT%.OneWireContainer1D;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankNVCRC;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.MemoryBankScratchEx;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer1F", "%CONT_ROOT%.OneWireContainer1F;%UTILS_ROOT%.Bit;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.SwitchContainer");
      temp.put("OneWireContainer20", "%CONT_ROOT%.OneWireContainer20;%UTILS_ROOT%.Bit;%CONT_ROOT%.ADContainer;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankAD;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.PagedMemoryBank");
      temp.put("OneWireContainer21", "%CONT_ROOT%.OneWireContainer21;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankNVCRC;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.MemoryBankScratchCRC;%CONT_ROOT%.MemoryBankScratchEx;%CONT_ROOT%.ClockContainer;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.TemperatureContainer;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer23", "%CONT_ROOT%.OneWireContainer23;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.MemoryBankNV;%CONT_ROOT%.MemoryBankScratch;%CONT_ROOT%.MemoryBankScratchEE;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.ScratchPad");
      temp.put("OneWireContainer26", "%CONT_ROOT%.OneWireContainer26;%CONT_ROOT%.ADContainer;%CONT_ROOT%.ClockContainer;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.TemperatureContainer");
      temp.put("OneWireContainer28", "%CONT_ROOT%.OneWireContainer28;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.TemperatureContainer");
      temp.put("OneWireContainer2C", "%CONT_ROOT%.OneWireContainer2C;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.PotentiometerContainer");
      temp.put("OneWireContainer30", "%CONT_ROOT%.OneWireContainer30;%CONT_ROOT%.ADContainer;%CONT_ROOT%.OneWireSensor;%CONT_ROOT%.TemperatureContainer");
      temp.put("OneWireContainer33", "%CONT_ROOT%.OneWireContainer33;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.OTPMemoryBank;%CONT_ROOT%.MemoryBankSHAEE");
      temp.put("OneWireMonitor",     "%UTILS_ROOT%.OneWireMonitor;%UTILS_ROOT%.OneWireMonitorEvent;%UTILS_ROOT%.OneWireMonitorEventListener");
      temp.put("NetworkMonitor",     "%UTILS_ROOT%.NetworkMonitor;%UTILS_ROOT%.NetworkMonitorEvent;%UTILS_ROOT%.NetworkMonitorEventListener;%UTILS_ROOT%.NetworkDeviceState;%UTILS_ROOT%.OWPath;%UTILS_ROOT%.OWPathElement");
      temp.put("OWFile",             "%UTILS_ROOT%.OWFile;%UTILS_ROOT%.OWFileDescriptor;%UTILS_ROOT%.OWFileInputStream;%UTILS_ROOT%.OWFileOutputStream;%UTILS_ROOT%.MemoryCache;%CONT_ROOT%.MemoryBank;%CONT_ROOT%.PagedMemoryBank;%CONT_ROOT%.OTPMemoryBank;%UTILS_ROOT%.Bit");
      temp.put("Thermometers",       "%OneWireContainer10%;%OneWireContainer21%;%OneWireContainer26%;%OneWireContainer28%;%OneWireContainer30%");
      temp.put("Clocks",             "%OneWireContainer04%;%OneWireContainer21%;%OneWireContainer26%");
      temp.put("Switches",           "%OneWireContainer05%;%OneWireContainer12%;%OneWireContainer1F%");
      temp.put("Potentiometers",     "%OneWireContainer2C%");
      temp.put("ADCs",               "%OneWireContainer20%;%OneWireContainer26%;%OneWireContainer30%");
      temp.put("AllMemoryDevices",   "%OneWireContainer04%;%OneWireContainer06%;%OneWireContainer08%;%OneWireContainer09%;%OneWireContainer0A%;%OneWireContainer0B%;%OneWireContainer0C%;%OneWireContainer0F%;%OneWireContainer12%;%OneWireContainer13%;%OneWireContainer14%;%OneWireContainer18%;%OneWireContainer1A%;%OneWireContainer1D%;%OneWireContainer20%;%OneWireContainer21%;%OneWireContainer23%;%OneWireContainer33%");
      temp.put("SHADebit",           "%OneWireContainer18%;%SHA_ROOT%.SHAiButtonUser;%SHA_ROOT%.SHAiButtonUser18;%OneWireContainer33%;%SHA_ROOT%.SHAiButtonUser;%SHA_ROOT%.SHAiButtonUser33;%SHA_ROOT%.SHATransaction;%SHA_ROOT%.SHADebit;%SHA_ROOT%.SHADebitUnsigned;%SHA_ROOT%.SHAiButtonCopr;%SHA_ROOT%.SHAiButtonCoprVM;%UTILS_ROOT%.SHA;%UTILS_ROOT%.IOHelper;%OWFile%");
      temp.put("Humidity",           "%OneWireContainer26%");
      temp.put("PROTOCOL",           "com.dalsemi.protocol");
      temp.put("PROTOCOL_BASE",      "%PROTOCOL%.BasicClient;%PROTOCOL%.DefaultFileNameMap;%PROTOCOL%.HeaderManager;");
      temp.put("COMM",               "com.dalsemi.comm");
      temp.put("SYSTEM",             "com.dalsemi.system");
      temp.put("TININET_HTTP",       "com.dalsemi.tininet.http");
      temp.put("FILE",               "%PROTOCOL_BASE%;%PROTOCOL%.URLDecoder;%PROTOCOL%.file.Connection;%PROTOCOL%.file.Handler;");
      temp.put("HTTP",               "%PROTOCOL%.http.Connection;%PROTOCOL%.http.Handler;%PROTOCOL_BASE%;%PROTOCOL%.http.HTTPClient;%PROTOCOL%.http.HTTPOutputStream");
      temp.put("FTP",                "%HTTP%;%PROTOCOL%.URLDecoder;%PROTOCOL%.ftp.FTPClient;%PROTOCOL%.ftp.Connection;%PROTOCOL%.ftp.Handler;%PROTOCOL_BASE%");
      temp.put("MAILTO",             "%PROTOCOL%.mailto.MailToPrintStream;%PROTOCOL%.mailto.Connection;%PROTOCOL%.mailto.Handler;%PROTOCOL%.URLDecoder;%PROTOCOL_BASE%");
      temp.put("HTTPSERVER",         "%TININET_HTTP%.HTTPServer;%TININET_HTTP%.HTTPServerException;%TININET_HTTP%.HTTPWorker;");
      temp.put("FTPCLIENT",          "%PROTOCOL%.ftp.FTPClient;%PROTOCOL%.BasicClient;%PROTOCOL%.URLDecoder;");
      temp.put("I2C",                "%SYSTEM%.I2CPort;(mod_i2c.tlib);");
      temp.put("CAN",                "%COMM%.CanBus;%COMM%.CanBusException;%COMM%.CanFrame;(mod_can.tlib);");
      temp.put("PPP",                "(ppp.tlib);%PPPBASE%.Chap;%PPPBASE%.Ipcp;%PPPBASE%.Lcp;%PPPBASE%.MD5;%PPPBASE%.Option;%PPPBASE%.Pap;%PPPBASE%.PPP;%PPPBASE%.PPPEvent;%PPPBASE%.PPPEventListener;%PPPBASE%.Protocol;%PPPBASE%.ProtocolEvent;%PPPBASE%.ProtocolEventListener;%PPPBASE%.Timer;%PPPBASE%.TimerEventListener;%PPPBASE%.Timer$TimerClient");


      dependency = temp;
   }

   private static String[] getDependencies (String hex)
   {
      if (dependency == null)
         return new String [0];

      //hex should be a 2 digit hex string
      String line = ( String ) dependency.get(hex);
      if (line==null)
      {
        //return new String[0];
        throw new RuntimeException("Could not find dependency list for "+hex);
      }

      line = replaceVariables(line);

      StringTokenizer st   = new StringTokenizer(line,";,");
      int             len  = st.countTokens();
      String[]        temp = new String [len];
      int             i    = 0;

      while (st.hasMoreTokens())
      {
         temp [i++] = st.nextToken();
      }

      return temp;
   }

   //replaces %variable% with its definition
   //CAUTION - we could get stuck in an infinite loop here!

   static Object lock = new Object();
   static boolean replaceDone;

   static String replaceVariables(String x)
   {
      String old = new String(x);
      replaceDone = false;

      Thread watchdog = new WatchDog(old);
      watchdog.start();

      int marker_index = x.indexOf('%');
      while (marker_index!=-1)
      {
          int second_index = x.indexOf('%', marker_index+1);
          if (second_index==-1)
          {
              throw new RuntimeException("Problem in the dependency file: Unbalanced '%' character!");
          }
          String key = x.substring(marker_index+1, second_index);
          String replacer = (String)dependency.get(key);
          x = x.substring(0,marker_index) + replacer + x.substring(second_index+1);

          marker_index = x.indexOf('%');
      }
      synchronized (lock)
      {
        replaceDone = true;
      }
      try
      {
        watchdog.join();
      }
      catch(InterruptedException ie)
      {
        //oh well - just let the thread die on its own
      }
      return x;
   }





   static String fileCheck(String filename) throws Exception
   {
       if (DEBUG)
            System.out.println("filecheck on "+filename);


       //this needs to be a loop through pathNames, returning if we find it, and then check
       //the current directory if we've exhausted all of that!!!

       String old_filename = filename;
       for (int i=0;i<pathNames.size();i++)
       {
           try
           {
                String pathname = (String)pathNames.elementAt(i);
                File f = new File(pathname);
                //we already know 'pathname' exists as a directory or a file

                if (f.isDirectory())
                {
                    //look for the file in the directory
                    filename = pathname + old_filename;
                    f = new File(filename);
                    if (f.exists())
                    {
                        if (DEBUG)
                            System.out.println("Loading "+old_filename+" from directory "+pathname);
                        return filename;
                    }
                    //else it doesn't exist, go through the loop again
                }
                else  //'pathname' must be a file, assume it's a jar/zip file
                {
                    filename = old_filename;
                    //first check the jar file
                    try
                    {
                        //open the zip and look for the entry
                        ZipFile zip = new ZipFile(pathname);
                        filename = filename.replace('/','_');
                        filename = filename.replace('\\','_');

                        f = new File(filename);
                        while (f.exists())
                        {
                            filename = "_"+filename;
                            f = new File(filename);
                        }
                        //now filename is the name of the file we will write to
                        ZipEntry ze = zip.getEntry(old_filename);
                        if (!ze.isDirectory())
                        {
                            filesToBeDeleted.addElement(filename);
                            FileOutputStream fout = new FileOutputStream(f);

                            InputStream fin = zip.getInputStream(ze);

                            int amount_read = fin.read(buffer);
                            while (amount_read != -1)
                            {
                                fout.write(buffer,0,amount_read);
                                amount_read = fin.read(buffer);
                            }

                            fin.close();
                            fout.close();
                            zip.close();
                            if (DEBUG)
                                System.out.println("Loading "+old_filename+" from zip file "+pathname);
                            return filename;
                        }
                    }
                    catch(Exception e1)
                    {
                        //drain it and go on to the next pathname in the loop
                    }
                }
           }
           catch(Exception e)
           {
                //drain it and go on to the next iteration of the loop
           }
       }  //end monster for loop

       //if we fall out to here it means we didn't find it in the path provided by the user,
       //so lets look in the current directory
       try
       {
            File f = new File(old_filename);
            if (f.exists())
            {
                if (DEBUG)
                    System.out.println("Loading "+old_filename+" from the current directory.");
                return old_filename;
            }
            //else fall out...
       }
       catch(Exception e)
       {
           //drain, it means we were unsuccesful
       }
       //we were unsuccessful. tell the main to clean up

       throw new Exception("Could not find the file "+filename);
   }

   public static void printUsageString()
   {
         System.out.println("Dallas Semiconductor TINI Program Building Tool");
         System.out.println("BuildDependency "+VERSION+", "+DATE+" (KLA)");
         System.out.println("--------------------------------------------------------------\r\n");
         System.out.println("This program is meant to be used to build applications for TINI");
         System.out.println("where building one class into a TINI application means that many");
         System.out.println("other classes (dependency classes) must be built in, such as programs");
         System.out.println("that use the 1-Wire API.  See the BuildDependency_README for more");
         System.out.println("information on using this program.\r\n\r\n");
         System.out.println("Usage: ");
         System.out.println("   java BuildDependency [options]\r\n");
         System.out.println("options: -add NAMES  -- The names of the dependencies to add to this");
         System.out.println("                        project. This is a semi-colon or comma separated");
         System.out.println("                        list of dependency names.");
         System.out.println("         -p PATH     -- Path to your dependency classes.  This is a");
         System.out.println("                        semi-colon or comma separated list that can");
         System.out.println("                        include multiple jar files and directories.");
         System.out.println("         -x DEP_FILE -- A semi-colon or comma separated list of filenames");
         System.out.println("                        of dependency text database files--multiple files");
         System.out.println("                        can now be specified, although note that if a key");
         System.out.println("                        is redefined, only the last definition found will");
         System.out.println("                        be used by BuildDependency--the keys won't be added.");
         System.out.println("                        BuildDependency has a default set of dependencies");
         System.out.println("                        used for 1-Wire programs.  See the readme for more");
         System.out.println("                        on this file's format.");
         System.out.println("         -debug      -- See the entire output of TINIConvertor");
         System.out.println("         -dep        -- See the entire dependency list using the specified");
         System.out.println("                        depdendency file or the default.");
         System.out.println("         -depNAME    -- See the depdendency list for dependency named NAME.");
         System.out.println("         -Any TINIConvertor Option");
         System.out.println("                     -- BuildDependency supports all TINIConvertor");
         System.out.println("                        options by directly passing any other options to");
         System.out.println("                        TINIConvertor.");
         System.out.println();
         System.out.println("For example, for a program that uses 1-Wire Containers for the DS1920,");
         System.out.println("DS1921, and the DS1996, I might try running: (EOLN's added for clarity)");
         System.out.println("   java BuildDependency -f ReadTemp.class -o readtemp.tini");
         System.out.println("           -d d:\\tini1.01\\bin\\tini.db -p OneWire.jar");
         System.out.println("           -add OneWireContainer10;OneWireContainer21;OneWireContainer0C");
         System.out.println();
         System.out.println("************************************************************************");
         System.out.println("* NOTE: 1. Dependency classes must already be compiled.                *");
         System.out.println("************************************************************************");
   }

}

class WatchDog extends Thread
{
    long start;
    final long TIMEOUT = 5000;
    String violation;

    public WatchDog(String x)
    {
        start = System.currentTimeMillis();
        violation = x;
    }

    public void run()
    {
        boolean done = false;
        while (!done)
        {
            if (System.currentTimeMillis() > start + TIMEOUT)
            {
                System.out.println("The watchdog has detected an infinite loop");
                System.out.println("This string is the guilty party:");
                System.out.println("    "+violation);
                System.out.println("Please remove the infinte loop from this string.");
                System.out.println("\r\nForcing program termination...");
                System.exit(0);
            }

            try
            {
                Thread.sleep(10);
            }
            catch(InterruptedException ie)
            {
            }

            synchronized (BuildDependency.lock)
            {
                done = BuildDependency.replaceDone;
            }
        }
    }
}
