#ifndef _A390_H_
#define _A390_H_

/* ----------------------------------------------------------------------
 * FILE: a390.h
 * PACKAGE: as31 - 8031/8051 Assembler.
 *
 * DESCRIPTION:
 *	The sole header file for the 8031/8051 assembler package.
 *	It defines several structures used by the yacc stack.
 *	It defines several macros for testing the bitsize of numeric
 *	quantities.
 *
 *	Some macros to extract information from the mode structure.
 *
 * REVISION HISTORY:
 *	Jan. 19, 1990 - Created. (Ken Stauffer)
 *
 * AUTHOR:
 *	All code in this file written by Ken Stauffer (University of Calgary).
 *	January, 1990.
 *
 */
/**/
/* ----------------------------------------------------------------------
 * user / keyword symbol structures:
 */

typedef struct 
{
  int size;
  char name[80];
  int  lcalls,
       ljmps,
       movazero,
       xrlazero,
       xrlaff;
} MODULE_STRUCT, *LPMODULE_STRUCT;

#define MAX_MODULE_STATS 100

typedef struct
{
  int count;
  MODULE_STRUCT modules[MAX_MODULE_STATS];
} MODULE_STATS, *LPMODULE_STATS;

struct opcode 
{
  char          *name;
  int           type;
  unsigned char *bytes;
};

struct symbol 
{
  char          *name;
  int           type;
  long          value;
  struct symbol *next;
};

#define UNDEF	0
#define LABEL	1

/* ----------------------------------------------------------------------
 * addressing mode stuff:
 */

struct mode 
{
  unsigned char mode;  /* value to index with */
  unsigned char size;  /* # of bytes used */
  unsigned char orval; /* value OR'd to obcode */
  unsigned char byte1; /* extra byte 1 */
  unsigned char byte2; /* extra byte 2 */
  unsigned char byte3; /* extra byte 3 */
};

#define set_md(m,a) ((m).mode   = (unsigned char)(a))
#define set_sz(m,a) ((m).size   = (unsigned char)(a))
#define set_ov(m,a) ((m).orval  = (unsigned char)(a))
#define set_b1(m,a) ((m).byte1  = (unsigned char)(a))
#define set_b2(m,a) ((m).byte2  = (unsigned char)(a))
#define set_b3(m,a) ((m).byte3  = (unsigned char)(a))

#define get_md(m) ((m).mode)
#define get_sz(m) ((m).size)
#define get_ov(m) ((m).orval)
#define get_b1(m) ((m).byte1)
#define get_b2(m) ((m).byte2)
#define get_b3(m) ((m).byte3)

/* ----------------------------------------------------------------------
 * yacc stack stuff:
 */

struct value 
{
  long          v;
  unsigned char d;		/* expression defined flag */
};

union ystack 
{
  long          value;
  struct value  val;
  struct opcode *op;
  struct symbol *sym;
  struct mode   mode;
  char          *str;
};

/* ----------------------------------------------------------------------
 * IS_BIT_MAPPED_RAM:
 *	True is the byte 'a' is the byte address of a bit mapped
 *	RAM location.
 */
#define isbmram(a) (((a) & 0xf0) == 0x20)

/* ----------------------------------------------------------------------
 * IS_BIT_MAPPED_SFR:
 *	True is the byte 'a' is the byte address of a bit mapped
 *	SPECIAL FUCTION REGISTER.
 */
#define isbmsfr(a) (((a) & 0x80) && !((a) & 0x07))

/* ----------------------------------------------------------------------
 * isbit8, ...
 *	Macros to check the sizes of values and to convert
 *	a value to a certain, size.
 *
 */
#define size8  (~0x0000ff)
#define size11 (~0x0007ff)
#define size13 (~0x001fff)
#define size16 (~0x00ffff)
#define size19 (~0x07ffff)
#define size24 (~0xffffff)

#define size10 (~0x03ff)
#define size12 (~0x0fff)
#define size15 (~0x7fff)
#define size18 (~0x3ffff)
#define size23 (~0x7fffff)

#define isbit8(v)  (!(((v) >= 0) ? (v)  & size8   : -(v)   >=  128))
#define isbit11(v) (!(((v) >= 0) ? (v)  & size11  : (-(v))  & size10))
#define isbit13(v) (!(((v) >= 0) ? (v)  & size13  : (-(v))  & size12))
#define isbit16(v) (!(((v) >= 0) ? (v)  & size16  : (-(v))  & size15))
#define isbit19(v) (!(((v) >= 0) ? (v)  & size19  : (-(v))  & size18))
#define isbit24(v) (!(((v) >= 0) ? (v)  & size24  : (-(v))  & size23))
                                            
/* ----------------------------------------------------------------------
 * Size of user hash table.
 */
#define HASHTABSIZE		1000

/* ----------------------------------------------------------------------
 * Macros to nicely test which pass we are in.
 */
#define pass1			(!pass)
#define pass2			(pass)

/* -------- TOKENS ------------------------------------------------------
 *
 * This includes the header file generated by yacc -d.
 * NOPE is defined inside of as31.y, which does not
 * need to re-include the tokens twice, thus NOPE prevents this.
 *
 */
#include  <stdio.h>
//#include  <stdlib.h>
#include  <string.h>
#include  <ctype.h>
#ifdef NOPE
#else
// hacked this in here, not recognized in .y file TAC
#define YYSTYPE union ystack
#include "Ytab.h"
#endif

void error(char *cs, ...);


#endif//_A390_H_
