/* ----------------------------------------------------------------------
 * FILE: as31.y
 * PACKAGE: as31 - 8031/8051 Assembler.
 *
 * DESCRIPTION:
 *  This file contains the yacc parser for the assembler.
 *  Related to this are the following:
 *    error(), warning(), yyerror()
 *    genbyte(), genword(), genstr(), makeop()
 *
 *
 * REVISION HISTORY:
 *  Jan. 19, 1990 - Created. (Ken Stauffer)
 *
 * AUTHOR:
 *  Original code in this file written by Ken Stauffer (University of Calgary).
 *  January 1990.
 *  24 bit addressing extensions written by Tom Chenot (Dallas Semiconductor).
 *  1999.
 *
 */

/* BUILD INFORMATION:
     BISON 1.27
      - NOTE: - In Windows you must point the environment variable 
                BISON_SIMPLE to the location of the file bison.simple .
              - It was difficult to compile the Bison source to run on
                Windows and it was a long, long time ago in a far, 
                far away place. It is completely up to the developer 
                to build and install Bison.

     GCC 2.91.66 or MSDEVSTUDIO/VC 6 
 */

/* LIMITATIONS:
     - see a390.l
 */

%{
//#define YYSTYPE union ystack

#include <setjmp.h>
#include <stdio.h>
#include <stdarg.h>
#define NOPE
#include "a390.h"
#undef NOPE
;
#define YYDEBUG 1 //tac
#define YYERROR_VERBOSE
#define YYSTYPE union ystack
;
#define UNKNOWN_TYPE 0
#define SYMBOL_TYPE  1
#define VALUE_TYPE   2
;
extern int      lineno;
extern int      dashl;
extern char     *asmfile;
extern char     *yytext;
extern char     g_CurrentIdentifier[];
extern jmp_buf  main_env;
extern FILE     *listing;
extern int      g_EndOfFile;
extern char     g_ErrorsEncountered;
extern char     g_Native;
extern char     g_StatsOn;
extern MODULE_STATS g_ModuleStats;
;
extern void emitbyte(int b);
extern void emitaddr(unsigned long int a);
extern void SymbolDump(FILE *p_ListFile);
extern void ResetLineBuffer();
extern void AddStringEntry(char *p_lpString);
extern void AddAFixup(void);
extern void AddLFixup(void);
extern void AddDFixup(void);
extern void Cleanup(void);
;
void  error(char *cs, ...);
void  warning(char *cs, ...);
void  yyerror(char *s);
int   makeop(struct opcode *op, struct mode *m, int add);
void  inclc(int i);
char  *padline(char *line);
void  dumplist(char *txt, int show);
void  genbyte(int b);
void  genstr(char *s);
void  genword(unsigned long w);
void  gentriple(unsigned long t);
;
int           pass,
              fatal;
unsigned long lc,
              cur_lc;

static unsigned char debugLow = 0;

//char debug_TEXT[10000];
char  g_CallOperation = 0;
char  g_ClrOperation = 0;

static unsigned char bytebuf[1024];    /* used by dumplist() */
static unsigned char debugHigh = 0;
static int bytecount;
static int g_LastType = UNKNOWN_TYPE;
/*
  statistical stuff
*/
int g_LCallCount     = 0;
int g_LJmpCount      = 0;
int g_MovAZeroCount  = 0;
int g_XrlAZeroCount  = 0;
int g_XrlAFFCount  = 0;
/**/
MODULE_STATS g_ModuleStats = {-1};

#ifndef YYSTYPE
#define YYSTYPE
#endif
;
/* ------------------------ G R A M M E R ----------------------------- */

%}

%token STRING
%token D_ORG
%token D_BYTE
%token D_WORD
%token D_SKIP
%token D_EQU
%token D_FLAG
%token D_END
%token ACALL
%token ADD
%token ADDC
%token AJMP
%token ANL
%token CJNE
%token CLR
%token CPL
%token DA
%token DEC
%token DIV
%token DJNZ
%token INC
%token JB
%token JBC
%token JC
%token JMP
%token JNB
%token JNC
%token JNZ
%token JZ
%token LCALL
%token LJMP
%token MOV
%token MOVC
%token MOVX
%token NOP
%token MUL
%token ORL
%token POP
%token PUSH
%token RET
%token RETI
%token RL
%token RLC
%token RR
%token RRC
%token SETB
%token SJMP
%token SUBB
%token SWAP
%token XCH
%token XCHD
%token XRL
%token AB
%token A
%token C
%token PC
%token DPTR
%token BITPOS
%token R0
%token R1
%token R2
%token R3
%token R4
%token R5
%token R6
%token R7
%token VALUE
%token SYMBOL
%token SEGMENT
%token SEGMENT_TYPE
%token D_TRIPLE
%token MODULE_BEGIN
%token MODULE_END 

%left '+' '-'
%left '*' '/' '%'
%left '|' '&'

%start program

%%
program     : linelist              { /* program */
                                      cur_lc = lc;
                                      return 0;
                                    }
            ;

linelist    : line                  { /* linelist */ }
            ;

line        : undefsym ':' linerest { /* line 1 */ 
                                      if(pass2)
                                        AddStringEntry(g_CurrentIdentifier);

                                      if(pass1) 
                                      {
                                        $1.sym->type  = LABEL;
                                        $1.sym->value = lc;
                                      }

                                      inclc($3.value);
                                      bytecount = 0;
                                    }
            | linerest              {  /* line 2 */ 
                                      inclc($1.value); 
                                      bytecount = 0; 
                                    }
            ;

linerest    : Segment
            | directive '\n'        { /* linerest directive */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);
                                    }
            | directive '\r' '\n'   { /* linerest directive */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);
                                    }
            | IntelDirective '\n'   { /* linerest IntelDirective */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);
                                    }
            | IntelDirective '\r' '\n'   { /* linerest IntelDirective */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);
                                    }
            | instr '\n'            { /* linerest instr */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);

                                      g_LastType = UNKNOWN_TYPE;
                                    }
            | instr '\r' '\n'       { /* linerest instr */ 
                                      $$.value = $1.value;

                                      if(dashl && pass2)
                                        dumplist($2.str, 1);

                                      g_LastType = UNKNOWN_TYPE;
                                    }
            | '\n'                  { /* linerest newline */ 
                                      $$.value = 0;

                                      if(dashl && pass2)
                                        dumplist($1.str, 0);
                                    }
            | '\r' '\n'             { /* linerest newline */ 
                                      $$.value = 0;

                                      if(dashl && pass2)
                                        dumplist($1.str, 0);
                                    }
            | 
            ;

/* --------------------
 * DIRECTIVES:
 *
 */
Segment     : SYMBOL SEGMENT SEGMENT_TYPE
                                    {
                                      $$.value = 0;
                                    }
            ;

IntelDirective : '$' SYMBOL         { /* IntelDirective 1 */ 
                                      $$.value = 0;
                                    }
            | '$' SYMBOL '(' SYMBOL ')'
                                    { /* IntelDirective 2 */ 
                                      $$.value = 0;
                                    }
            | '$' SYMBOL '(' SYMBOL '.' INC ')'
                                    { /* IntelDirective 3 */ 
                                      $$.value = 0;
                                    }
            ;

directive   : D_ORG defexpr         { /* directive 1 */
                                      if(g_Native)
                                      {
                                        error("Absolute addressing is not available", NULL);
                                      }
                                      else
                                      {
                                        lc = $2.val.v;

                                        if(pass2) 
                                          emitaddr(lc);

                                        bytecount = 0;
                                        $$.value  = 0;
                                      }
                                    }
            | D_BYTE blist          { /* directive 2 */  
                                      $$.value = $2.value; 
                                    }
            | D_WORD wlist          { /* directive 3 */  
                                      $$.value = $2.value; 
                                    }
            | D_TRIPLE tlist         { /* directive 3 */  
                                      $$.value = $2.value; 
                                    }
            | '.' D_SKIP defexpr    { /* directive 4 */  
                                      $$.value = $3.val.v;

                                      if(pass2)
                                        emitaddr(lc + $$.value); 
                                    }
            | undefsym D_EQU  expr
                                    { /* directive 5 */ 
                                    /*
                                      if($3.val.d == 0)
                                        error("Expression is undefined in pass 1", NULL);
                                    */
                                      $1.sym->type  = LABEL;
                                      $1.sym->value = $3.val.v;
                                      $$.value      = 0;
                                    }  
            | '.' D_FLAG SYMBOL ',' flag
                                    { /* directive 6 */ 
                                      $3.sym->type  = LABEL;
                                      $3.sym->value = $5.value;
                                      $$.value      = 0;
                                    }
            | D_END                 { /* directive 7 */  
                                      $$.value    = 0;
                                      g_EndOfFile = 1;
                                    }
            ;

defexpr     : expr                  { /* defexpr */ 
                                      if($1.val.d == 0)
                                        error("Expression is undefined in pass 1", NULL);

                                      if(!isbit24($1.val.v))
                                      //if(!(isbit16($1.val.v)))
                                        error("Value greater than 16-bits", NULL);

                                      $$.value = $1.val.v;
                                    }
            ;

flag        : flagv BITPOS          { /* flag */ 
                                      if(!isbit8($1.value))
                                        warning("Bit address exceeds 8-bits", NULL);

                                      if(isbmram($1.value))
                                        $$.value = ($1.value  - 0x20) * 8 + $2.value;
                                      else if(isbmsfr($1.value))
                                        $$.value = $1.value + $2.value;
                                      else
                                        warning("Invalid bit addressable RAM location", NULL);
                                    }
            ;

flagv       : SYMBOL                { /* flagv 1 */ 
                                      if($1.sym->type == UNDEF)
                                        error("Symbol %s must be defined in pass 1", $1.sym->name, NULL);

                                      $$.value = $1.sym->value;
                                    }
            | VALUE                 { /* flagv 2 */  $$.value = $1.value; }
            ;

undefsym    : SYMBOL                { /* undefsym */ 
                                      if($1.sym->type != UNDEF && pass1)
                                        error("Attempt to redefine symbol: %s", $1.sym->name, NULL);

                                      $$.sym = $1.sym;
                                    }
            ;

blist       : blist ',' data8       { /* blist 1 */ 
                                      if(pass2) 
                                        genbyte($3.value);

                                      $$.value = $1.value + 1;
                                    }
            | blist ',' STRING      { /* blist 2 */ 
                                      if(pass1)
                                        $$.value = $1.value + $3.value;
                                      else  
                                      { 
                                        $$.value = $1.value + strlen($3.str);
                                        genstr($3.str);
                                        
                                        free($3.str);
                                      }
                                    }
            | data8                 { /* blist 3 */ 
                                      if(pass2) 
                                        genbyte($1.value);

                                      $$.value = 1;
                                    }
            | STRING                { /* blist 4 */ 
                                      if(pass1)
                                        $$.value = $1.value;
                                      else 
                                      {
                                        $$.value = strlen($1.str);
                                        genstr($1.str);
                                        free($1.str);
                                      }
                                    }
            ;

wlist       : wlist ',' data16      { /* wlist 1 */ 
                                      if(pass2) 
                                        genword($3.value);

                                      $$.value = $1.value + 2;
                                    }
            | data16                { /* wlist 2 */ 
                                      if(pass2) 
                                        genword($1.value);

                                      $$.value = 2;
                                    }
            ;
tlist       : tlist ',' data24      { /* wlist 1 */ 
                                      if(pass2) 
                                        gentriple($3.value);

                                      $$.value = $1.value + 3;
                                    }
            | data24                { /* wlist 2 */ 
                                      if(pass2) 
                                        gentriple($1.value);

                                      $$.value = 3;
                                    }
            ;
/* --------------------
 * EXPRESSIONS:
 *
 */

expr        : '(' expr ')'          { /* expr 2 */  
                                      $$.val.v = $2.val.v;
                                      $$.val.d = $2.val.d; 
                                    }
            | '-' expr %prec '*'    { /* expr 3 */  
                                      $$.val.v = -$2.val.v;
                                      $$.val.d = $2.val.d;  
                                    }
            | expr '|' expr         { /* expr 4 */  
                                      $$.val.v = $1.val.v |   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '&' expr         { /* expr 5 */  
                                      $$.val.v = $1.val.v &   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '*' expr         { /* expr 6 */  
                                      $$.val.v = $1.val.v *   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '/' expr         { /* expr 7 */  
                                      $$.val.v = $1.val.v /   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '%' expr         { /* expr 8 */  
                                      $$.val.v = $1.val.v %   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '-' expr         { /* expr 9 */  
                                      $$.val.v = $1.val.v -   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | expr '+' expr         { /* expr 10 */  
                                      $$.val.v = $1.val.v +   $3.val.v;
                                      $$.val.d = $1.val.d &&  $3.val.d; 
                                    }
            | SYMBOL                { /* expr 11 */ 
                                      if(pass1) 
                                      {
                                        $$.val.v = $1.sym->value;
                                        $$.val.d = ($1.sym->type != UNDEF);
                                      }
                                      else 
                                      {
                                        if($1.sym->type == UNDEF)
                                          error("Undefined symbol %s", $1.sym->name, NULL);

                                        $$.val.v = $1.sym->value;
                                        $$.val.d = 1;
                                      }

                                      g_LastType = SYMBOL_TYPE;
                                    }
            | VALUE                 { /* expr 12 */  
                                      $$.val.v  = $1.val.v; 
                                      $$.val.d  = 1; 
                                      g_LastType = VALUE_TYPE;
                                    }
            | '$'                   { /* expr 13 */  
                                      $$.val.v  = cur_lc; 
                                      $$.val.d  = 1; 
                                    }
            ;
/* --------------------
 * INSTRUCTIONS:
 *
 */

instr       : NOP                   { /* instr - NOP */         
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                           
            | ACALL addr19          { /* instr - ACALL */       
                                      if(g_Native)
                                      {
                                        error("ACALL is not supported in this mode", NULL);
                                      }
                                      else
                                      {
                                        if(pass2)
                                          if(g_LastType != VALUE_TYPE)
                                            AddAFixup();

                                        $$.value = makeop($1.op,  &$2.mode, 0); 
                                        g_CallOperation = 1;
                                      }
                                    }                                     
            | AJMP addr19           { /* instr - AJMP */                  
                                      if(g_Native)
                                      {
                                        error("AJMP is not supported in this mode", NULL);
                                      }
                                      else
                                      {
                                        if(pass2)
                                          if(g_LastType != VALUE_TYPE)
                                            AddAFixup();

                                        $$.value = makeop($1.op,  &$2.mode, 0); 
                                      }
                                    }                                     
            | ADD two_op1           { /* instr - ADD */                   
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | ADDC two_op1          { /* instr - ADDC */                  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | SUBB two_op1          { /* instr - SUBB */                  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | XRL two_op1           { /* instr - XRL 1 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | XRL two_op2           { /* instr - XRL 2 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 4); 
                                    }                                     
            | ANL two_op1           { /* instr - ANL 1 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | ANL two_op2           { /* instr - ANL 2 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 4); 
                                    }                                     
            | ANL two_op3           { /* instr - ANL 3 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 6); 
                                    }                                     
            | ORL two_op1           { /* instr - ORL 1 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | ORL two_op2           { /* instr - ORL 2 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 4); 
                                    }                                     
            | ORL two_op3           { /* instr - ORL 3 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 6); 
                                    }                                     
            | XCH two_op1           { /* instr - XCH */  
                                      if(get_md($2.mode) == 3)
                                        error("Immediate mode is illegal", NULL);

                                      $$.value = makeop($1.op,  &$2.mode, 0);
                                    }                           
            | INC single_op1        { /* instr - INC 1 */       
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                           
            | INC DPTR              { /* instr - INC 2 */       
                                      $$.value = makeop($1.op,  NULL, 4); 
                                    }                           
            | DEC single_op1        { /* instr - DEC */         
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                           
            | DA A                  { /* instr - DA */                
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | DIV AB                { /* instr - DIV */               
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | JMP '@' A '+' DPTR    { /* instr - JMP 1 */             
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | JMP '@' DPTR '+' A    { /* instr - JMP 2 */             
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | MUL AB                { /* instr - MUL */               
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RET                   { /* instr - RET */               
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RETI                  { /* instr - RETI */              
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RL A                  { /* instr - RL */                
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RLC A                 { /* instr - RLC */               
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RR A                  { /* instr - RR */                
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | RRC A                 { /* instr - RRC */               
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | SWAP A                { /* instr - SWAP */              
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | XCHD two_op1          { /* instr - XCHD */  
                                      if(get_md($2.mode) != 2)
                                        error("Invalid addressing mode", NULL);

                                      $$.value = makeop($1.op,  &$2.mode, -2); 
                                    }
            | CLR single_op2        { /* instr - CLR */  
                                      $$.value = makeop($1.op,  &$2.mode, 0);
                                      g_ClrOperation = 1; 
                                    }
            | CPL single_op2        { /* instr - CPL */  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }
            | SETB single_op2       { /* instr - SETB */  
                                      if(get_md($2.mode) == 0)
                                        error("Invalid addressing mode", NULL);

                                      $$.value = makeop($1.op,  &$2.mode, -1); 
                                    }
            | PUSH data8            { /* instr - PUSH */ 
                                      struct mode tmp;

                                      set_md(tmp, 0);
                                      set_ov(tmp, 0);
                                      set_sz(tmp, 1);
                                      set_b1(tmp, $2.value);
                                      $$.value = makeop($1.op,  &tmp, 0);
                                    }
            | POP data8             { /* instr - POP */ 
                                      struct mode tmp;

                                      set_md(tmp, 0);
                                      set_ov(tmp, 0);
                                      set_sz(tmp, 1);
                                      set_b1(tmp, $2.value);
                                      $$.value = makeop($1.op,  &tmp, 0);
                                    }                           
            | LJMP addr24           { /* instr - LJMP */                  
                                      if(pass2)
                                      {
                                        if(g_LastType != VALUE_TYPE)
                                          AddLFixup();

                                        g_LJmpCount++;
                                        g_ModuleStats.modules[g_ModuleStats.count].ljmps++;
                                      }

                                      $$.value = makeop($1.op,  &$2.mode, 0);
                                    }
            | LCALL addr24          { /* instr - LCALL */                 
                                      if(pass2)
                                      {
                                        if(g_LastType != VALUE_TYPE)
                                          AddLFixup();

                                        g_LCallCount++;
                                        g_ModuleStats.modules[g_ModuleStats.count].lcalls++;
                                      }

                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                      g_CallOperation = 1;
                                    }                                     
            | JC relative           { /* instr - JC */                    
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JNC relative          { /* instr - JNC */                   
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JNZ relative          { /* instr - JNZ */                   
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JZ relative           { /* instr - JZ */                    
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | SJMP relative         { /* instr - SJMP */                  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | CJNE three_op1        { /* instr - CJNE */                  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JB two_op4            { /* instr - JB */                    
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JNB two_op4           { /* instr - JNB */                   
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | JBC two_op4           { /* instr - JBC */                   
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | DJNZ two_op5          { /* instr - DJNZ */                  
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | MOV two_op1           { /* instr - MOV 1 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 0); 
                                    }                                     
            | MOV two_op2           { /* instr - MOV 2 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 4); 
                                    }                                     
            | MOV two_op6           { /* instr - MOV 3 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 6); 
                                    }                                     
            | MOV three_op7         { /* instr - MOV 4 */                 
                                      $$.value = makeop($1.op,  &$2.mode, 7); 
                                    }                                     
            | MOVC A ',' '@' A '+' DPTR                         
                                    { /* instr - MOVC 1 */      
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | MOVC A ',' '@' DPTR '+' A                               
                                    { /* instr - MOVC 2 */            
                                      $$.value = makeop($1.op,  NULL, 0); 
                                    }                                 
            | MOVC A ',' '@' A '+' PC                                 
                                    { /* instr - MOVC 3 */            
                                      $$.value = makeop($1.op,  NULL, 1); 
                                    }                                 
            | MOVC A ',' '@' PC '+' A                                 
                                    { /* instr - MOVC 4 */            
                                      $$.value = makeop($1.op,  NULL, 1); 
                                    }                                 
            | MOVX A ',' '@' regi   { /* instr - MOVX 1 */            
                                      $$.value = makeop($1.op,  NULL, $5.value); 
                                    }                                 
            | MOVX A ',' '@' DPTR   { /* instr - MOVX 2 */            
                                      $$.value = makeop($1.op,  NULL, 2); 
                                    }                                 
            | MOVX '@' regi ',' A   { /* instr - MOVX 3 */            
                                      $$.value = makeop($1.op,  NULL, $3.value+3); 
                                    }                                 
            | MOVX '@' DPTR ',' A   { /* instr - MOVX 4 */            
                                      $$.value = makeop($1.op,  NULL, 5); 
                                    }                           
            ;                                                   
/* --------------------                                         
 * ADDRESSING MODES:                                            
 *                                                              
 */                                                             

two_op1     : A ',' reg
                                    { /* two_op1 */ 
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, $3.value);
                                      set_sz($$.mode, 0);
                                    }
            | A ',' data8
                                    { /*  */ 
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $3.value);
                                    }                 
            | A ',' '@' regi                          
                                    { /*  */          
                                      set_md($$.mode, 2);
                                      set_ov($$.mode, $4.value);
                                      set_sz($$.mode, 0);
                                    }                 
            | A ',' '#' data8                         
                                    { /*  */          
                                      set_md($$.mode, 3);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $4.value);

                                      if(pass2)
                                      {
                                        if(!strncmp($0.op->name,"mov",3))
                                        {
                                          if(!$4.value)
                                          {
                                            g_MovAZeroCount++;
                                            g_ModuleStats.modules[g_ModuleStats.count].movazero++;
                                          }
                                        }
                                        else if(!strncmp($0.op->name,"xrl",3))
                                        {
                                          if(!$4.value)
                                          {
                                            g_XrlAZeroCount++;
                                            g_ModuleStats.modules[g_ModuleStats.count].xrlazero++;
                                          }
                                          else if($4.value == 255)
                                          {
                                            g_XrlAFFCount++;
                                            g_ModuleStats.modules[g_ModuleStats.count].xrlaff++;
                                          }
                                        }
                                      }
                                    }                 
            ;                                         

two_op2     : data8 ',' A                             
                                    { /* two_op2 */   
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);
                                    }
            | data8 ',' '#' data8
                                    { /*  */ 
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $1.value);
                                      set_b2($$.mode, $4.value);
                                    }                 
            ;                                         

two_op3     : C ',' bit                               
                                    { /* two_op3 */   
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $3.value);
                                    }                 
            | C ',' '/' bit                           
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $4.value);
                                    }                 
            | C ',' '!' bit                           
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $4.value);
                                    }                 
            ;                                         

two_op4     : bit ',' rel                             
                                    { /* two_op4 */   
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $1.value);
                                      set_b2($$.mode, $3.value);
                                    }                 
            ;                                         

two_op5     : reg ',' rel2                            
                                    { /* two_op5 */   
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $3.value);
                                    }                 
            | data8 ',' rel                           
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $1.value);
                                      set_b2($$.mode, $3.value);
                                    }
            ;

two_op6     : reg ',' A
                                    { /* two_op6 */   
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 0);
                                    }                 
            | reg ',' data8                           
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $3.value);
                                    }                 
            | reg ',' '#' data8                       
                                    { /*  */          
                                      set_md($$.mode, 2);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $4.value);
                                    }                 
            | data8 ',' reg                           
                                    { /*  */          
                                      set_md($$.mode, 3);
                                      set_ov($$.mode, $3.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);
                                    }                 
            | data8 ',' data8                         
                                    { /*  */          
                                      set_md($$.mode, 4);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $3.value);
                                      set_b2($$.mode, $1.value);
                                    }                 
            | data8 ',' '@' regi                      
                                    { /*  */          
                                      set_md($$.mode, 5);
                                      set_ov($$.mode, $4.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);
                                    }                 
            | '@' regi ',' A                          
                                    { /*  */          
                                      set_md($$.mode, 6);
                                      set_ov($$.mode, $2.value);
                                      set_sz($$.mode, 0);
                                    }                 
            | '@' regi ',' data8                      
                                    { /*  */          
                                      set_md($$.mode, 7);
                                      set_ov($$.mode, $2.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $4.value);
                                    }                 
            | '@' regi ',' '#' data8
                                    { /*  */ 
                                      set_md($$.mode, 8);
                                      set_ov($$.mode, $2.value);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $5.value);
                                    }
            | C ',' bit
                                    { /*  */ 
                                      set_md($$.mode, 10);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $3.value);
                                    }
            ;

three_op7   :  DPTR ',' '#' data24
                                    { /* three_op7 */ 
                                      if(pass2)
                                        if(g_LastType != VALUE_TYPE)
                                          AddDFixup();

                                      // BUG BUG - this is a bug in the original code
                                      // The opcode is off by 1 which actually indexes 
                                      // outside the opcode array for mov
                                      //set_md($$.mode, 9);
                                      // HACK HACK - I don't fully understand the mode
                                      // but I'm reducing it by 1 to index the correct opcode
                                      set_md($$.mode, 8);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 3);
                                      set_b1($$.mode, ($4.value & 0xff0000) >> 16);
                                      set_b2($$.mode, ($4.value & 0x00ff00) >> 8);
                                      set_b3($$.mode, ($4.value & 0x0000ff));
                                    }
  /*
   * Following two productions cannot be represented by:
   *
   *  bit ',' C
   *
   * Because yacc gives tons of reduce/reduce errors if
   * that is attempted.
   *
   */
            | data8 ',' C
                                    { /* mov_bit_c 1 */ 
                                      // BUG BUG - this is a bug in the original code
                                      // The opcode is off by 1 which actually indexes 
                                      // outside the opcode array for mov
                                      //set_md($$.mode, 11);
                                      // HACK HACK - I don't fully understand the mode
                                      // but I'm reducing it by 1 to index the correct opcode
                                      set_md($$.mode, 10);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);
                                    }
            | data8 BITPOS ',' C
                                    { /* mov_bit_c 2 */ 
                                      if(pass2) 
                                      {
                                        if(!isbit8($1.value))
                                          warning("Bit address exceeds 8-bits", NULL);

                                        if(isbmram($1.value))
                                          set_b1($$.mode, ($1.value - 0x20) * 8 + $2.value);
                                        else if(isbmsfr($1.value))
                                          set_b1($$.mode, $1.value + $2.value);
                                        else
                                          warning("Invalid bit addressable RAM location", NULL);
                                      }

                                      // BUG BUG - this is a bug in the original code
                                      // The opcode is off by 1 which actually indexes 
                                      // outside the opcode array for mov
                                      //set_md($$.mode, 11);
                                      // HACK HACK - I don't fully understand the mode
                                      // but I'm reducing it by 1 to index the correct opcode
                                      set_md($$.mode, 10);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                    }
            ;

single_op1  : A
                                    { /* single_op1 */ 
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 0);
                                    }
            | reg
                                    { /*  */ 
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 0);
                                    }
            | data8
                                    { /*  */ 
                                      set_md($$.mode, 2);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);
                                    }
            | '@' regi
                                    { /*  */ 
                                      set_md($$.mode, 3);
                                      set_ov($$.mode, $2.value);
                                      set_sz($$.mode, 0);
                                    }                 
            ;

single_op2  : A
                                    { /* single_op2 */ 
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 0);
                                    }                 
            | C                                       
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 0);
                                    }                 
            | bit                                     
                                    { /*  */          
                                      set_md($$.mode, 2);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.value);

                                      /* 
                                        Okay, you whiners complained that I did not scream about
                                        clr ACC (which is equivalent to clr ACC.0). Here it is!
                                      */
                                      if(pass2)
                                      {
                                        if(g_ClrOperation)
                                        {
                                          g_ClrOperation = 0;

                                          if($1.value == 0xe0)
                                            if(*yytext != '.')
                                              fprintf(stderr, 
                                                      "!!!Warning, Implicit \"clr ACC.0\" detected at address 0x%06X!!!\n",
                                                      lc);
  }
}                                    }                 
            ;                                         

three_op1   : A ',' data8 ',' rel                     
                                    { /* three_op1 */  
                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $3.value);
                                      set_b2($$.mode, $5.value);
                                    }                 
            | A ',' '#' data8 ',' rel                 
                                    { /*  */          
                                      set_md($$.mode, 1);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $4.value);
                                      set_b2($$.mode, $6.value);
                                    }                 
            | reg ',' '#' data8 ',' rel               
                                    { /*  */          
                                      set_md($$.mode, 2);
                                      set_ov($$.mode, $1.value);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $4.value);
                                      set_b2($$.mode, $6.value);
                                    }                 
            | '@' regi ',' '#' data8 ',' rel          
                                    { /*  */          
                                      set_md($$.mode, 3);
                                      set_ov($$.mode, $2.value);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, $5.value);
                                      set_b2($$.mode, $7.value);
                                    }
            ;

rel         : expr
                                    { /* rel */ 
                                      long offset;

                                      if(pass2) 
                                      {
                                        offset = $1.val.v - (lc + 3);

                                        if(offset > 127 || offset < -128)
                                           warning("Target out of range. Relative offset exceeds -128 / +127", NULL);

                                        $$.value = offset;
                                      }
                                    }
            ;
/*
 * This production differs from the above, by 1 number!
 *
 */
rel2        : expr
                                    { /* rel2 */ 
                                      long offset;

                                      if(pass2) 
                                      {
                                        offset = $1.val.v - (lc + 2); /* different! */

                                        if(offset > 127 || offset < -128)
                                           warning("Target out of range. Relative offset exceeds -128 / +127", NULL);

                                        $$.value = offset;
                                      }
                                    }
            ;

bit         : bitv BITPOS
                                    { /* bit 1 */ 
                                      if(pass2) 
                                      {
                                        if(!isbit8($1.value))
                                          warning("Bit address exceeds 8-bits", NULL);

                                        if(isbmram($1.value))
                                          $$.value = ($1.value - 0x20) * 8 + $2.value;
                                        else if(isbmsfr($1.value))
                                          $$.value = $1.value + $2.value;
                                        else
                                          warning("Invalid bit addressable RAM location", NULL);
                                      }
                                    }
            | bitv
                                    { /* bit 2  */ 
                                      if(pass2) 
                                      {
                                        if(!isbit8($1.value))
                                          warning("Bit address exceeds 8-bits", NULL);

                                        $$.value = $1.value;
                                      }
                                    }
            ;

bitv        : SYMBOL
                                    { /* bitv 1 */ 
                                      if($1.sym->type == UNDEF && pass2)
                                        error("Symbol %s undefined", $1.sym->name, NULL);

                                      $$.value = $1.sym->value;
                                    }
            | VALUE                 { /* bitv 2 */  
                                      $$.value = $1.value; 
                                    }
            ;

reg         : R0                    { /* reg */  
                                      $$.value = 0; 
                                    }
            | R1                    { /*  */  
                                      $$.value = 1; 
                                    }
            | R2                    { /*  */  
                                      $$.value = 2; 
                                    }
            | R3                    { /*  */  
                                      $$.value = 3; 
                                    }
            | R4                    { /*  */  
                                      $$.value = 4; 
                                    }
            | R5                    { /*  */  
                                      $$.value = 5; 
                                    }
            | R6                    { /*  */  
                                      $$.value = 6; 
                                    }
            | R7                    { /*  */  
                                      $$.value = 7; 
                                    }
            ;

regi        : R0                    { /* regi */  
                                      $$.value = 0; 
                                    }
            | R1                    { /*  */  
                                      $$.value = 1; 
                                    }
            | R2
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r2", NULL);
                                    }
            | R3
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r3", NULL);
                                    }
            | R4
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r4", NULL); 
                                    }
            | R5
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r5", NULL); 
                                    }
            | R6
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r6", NULL);
                                    }
            | R7
                                    { /*  */  
                                      $$.value = 0;
                                      warning("Illegal indirect register: @r7", NULL);
                                    }
            ;

data8       : expr
                                    { /* data8 */ 
                                      if(pass2) 
                                      {
                                        //if(!isbit8($1.val.v))
                                          //warning("Expression greater than 8-bits", NULL);
                                      }

                                      $$.value = ($1.val.v & 0x0ff);
                                    }
            ;

data16      : expr
                                    { /* data16 */ 
                                      if(pass2) 
                                      {  
                                        //if(!isbit16($1.val.v))
                                          //warning("Expression greater than 16-bits", NULL);
                                      }

                                      $$.value = ($1.val.v & 0x0ffff);
                                    }
            ;

data24      : expr
                                    { /* data24 */ 
                                      if(pass2) 
                                      {
                                        //if(!isbit24($1.val.v))
                                          //warning("Expression greater than 24-bits", NULL);
                                      }

                                      $$.value = ($1.val.v & 0x0ffffff);
                                    }
            ;

addr11      : expr
                                    { /* addr11 */ 
                                      if(pass2) 
                                      {
                                  //      if(!isbit16($1.val.v))
                                  //        warning("Address greater than 16-bits", NULL);
                                        if(!isbit24($1.val.v))
                                          warning("Address greater than 24-bits", NULL);

                                        if(($1.val.v & size11) != ((lc + 2) & size11))
                                          warning("Address outside current 2K page", NULL);
                                      }

                                      set_md($$.mode, 0);
                                      set_ov($$.mode, ($1.val.v & 0x0700) >> 3);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, $1.val.v & 0x00ff);
                                    }
            ;

addr19      : expr
                                    { /* addr11 */ 
                                      if(pass2) 
                                      {
                                        if(!isbit24($1.val.v))
                                          warning("Address greater than 24-bits", NULL);

                                        if(($1.val.v & size19) != ((lc + 2) & size19))
                                          warning("Address outside current page", NULL);
                                      }

                                      set_md($$.mode, 0);
                                      set_ov($$.mode, ($1.val.v & 0x070000) >> 11);
                                      set_sz($$.mode, 2);
                                      set_b1($$.mode, ($1.val.v & 0x00ff00) >> 8);
                                      set_b2($$.mode, ($1.val.v & 0x0000ff));
                                    }
            ;

addr16      : expr
                                    { /* addr16 */
                                      if(pass2) 
                                      {
                                        if(!isbit16($1.val.v))
                                          warning("Address greater than 16-bits", NULL);

                                  //      if(!isbit24($1.val.v))
                                    //      warning("Address greater than 24-bits", NULL);
                                      }

                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 2);
                                      ////set_b1($$.mode, ($1.val.v & 0xff0000) >> 16);
                                      ////set_b2($$.mode, ($1.val.v & 0x00ff00) >> 8);
                                      set_b1($$.mode, ($1.val.v & 0x00ff00) >> 16);
                                      set_b2($$.mode, ($1.val.v & 0x0000ff) >> 8);
                                    }
            ;

addr24      : expr
                                    { /* addr24 */
                                      if(pass2) 
                                      {
                                        if(!isbit24($1.val.v))
                                          warning("Address greater than 24-bits", NULL);
                                      }

                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 3);
                                      set_b1($$.mode, ($1.val.v & 0x0ff0000) >> 16);
                                      set_b2($$.mode, ($1.val.v & 0x000ff00) >> 8);
                                      set_b3($$.mode, ($1.val.v & 0x00000ff));
                                    }
            ;

relative    : expr
                                    { /* relative */ 
                                      long offset;

                                      if(pass2) 
                                      {
                                        offset = $1.val.v - (lc + 2);

                                        if(offset>127 || offset<-128)
                                           warning("Target out of range. Relative offset exceeds -128 / +127", NULL);
                                      }

                                      set_md($$.mode, 0);
                                      set_ov($$.mode, 0);
                                      set_sz($$.mode, 1);
                                      set_b1($$.mode, offset);
                                    }
            ;

%%

/* ---------------------------------------------------------------------- */
/* ----------------------------------------------------------------------
 * InitPass:
 *  initializes variables
 */
void InitPass()
{
  memset(&bytebuf, 0x55, sizeof(bytebuf));
}
/**********************************************************************/
void yyerror(char *s)
{
  error(s, NULL);
}
/* ----------------------------------------------------------------------
 * error:
 *  Uses semi-variable arguments. This causes immediate assembler
 *  termination.
 */
void error(char *cs, ...)
{
  va_list l_Marker;
  char    *l_lpCurrentArg;

  fprintf(stderr, "File: %s, line: %d, ", asmfile, lineno);

  va_start(l_Marker, cs);     /* Initialize variable arguments. */

  l_lpCurrentArg = va_arg(l_Marker, char *);
  fprintf(stderr, cs, l_lpCurrentArg);

  va_end(l_Marker);              /* Reset variable arguments.      */

  fprintf(stderr, ".\n\tLast token read was \"%s\".\n",yytext);
  longjmp(main_env, 1);
}
/* ----------------------------------------------------------------------
 * warning:
 *  Produce error message. This will abort assembly at
 *  the end of the current pass.
 *
 */
void warning(char *cs, ...)
{
  va_list l_Marker;
  char    *l_lpCurrentArg;

  fatal++;
  fprintf(stderr, "File: %s, line: %d, ", asmfile, lineno);

  va_start(l_Marker, cs);     /* Initialize variable arguments. */
  l_lpCurrentArg = va_arg(l_Marker, char *);
  fprintf(stderr, cs, l_lpCurrentArg);

  va_end(l_Marker);              /* Reset variable arguments.      */

  fprintf(stderr, ".\n\tLast token read was \"%s\".\n",yytext);
  g_ErrorsEncountered = 1;
}
/* ----------------------------------------------------------------------
 * makeop:
 *  This function makes an opcode based on the instruction symbol table
 *  entry, and an addressing mode structure.
 *  This function is called from both passes, but
 *  only generates code in pass 2.
 *
 *  Resultant opcode bytes are passed to genbyte().
 *
 *  Returns the number of bytes that the instruction
 *  occupies.
 *
 */
int makeop(struct opcode *op, struct mode *m, int add)
{
  register unsigned int newop;

  if(m == NULL) 
  {
    if(pass2) 
      genbyte(op->bytes[0 + add]);

    return(1);
  }

  if(pass2) 
  {
    newop = op->bytes[get_md(*m) + add] | get_ov(*m);
    genbyte(newop);

    if(get_sz(*m) > 0) 
      genbyte(get_b1(*m));

    if(get_sz(*m) > 1) 
      genbyte(get_b2(*m));

    if(get_sz(*m) > 2) 
      genbyte(get_b3(*m));
  }

  return(get_sz(*m) + 1);
}
/* ----------------------------------------------------------------------
 * inclc:
 *  Increments the Location Counter by 'i' amount.
 *  Check to see if 'i' overflows.
 *  Checks to see if assembler is overlapping previous sections
 *  of code. (using a large bit field).
 *
 */
#define indx(a) ((a) / (sizeof(long) * 8))
#define bit(a)  (1 << ((a) % (sizeof(long) * 8)))

#define getloc(a) (regions[indx(a)] &   bit(a))
#define setloc(a) (regions[indx(a)] |=  bit(a))

void inclc(int i)
{
  static unsigned long regions[0x1000000/(sizeof(long)*8)];

  while(i-- > 0) 
  {
    if(pass2 && getloc(lc))
      error("Location counter overlaps", NULL);

    if(pass2) 
      setloc(lc);

    lc += 1;
  }
    
  if(lc > 0xffffff)
    error("Location counter has exceeded 24-bits", NULL);
}
/* ----------------------------------------------------------------------
 * padline:
 *  This routine returns a new string, which is equivilant to
 *  'line' except that all tabs have been expanded to spaces, and
 *  the total length has been truncated to 58 chars.
 */
char *padline(char *line)
{
  static char newline[60];
  char        *p1;
  int         pos = 0,
              nxtpos;

  for(p1 = line; pos < sizeof(newline) - 1 && *p1; p1++) 
  {
    if(*p1 == '\t') 
    {
      nxtpos = pos + 8 - pos % 8;

      while(pos < sizeof(newline) - 1 && pos <= nxtpos)
        newline[pos++] = ' ';
    } 
    else if(*p1 != '\n')
      newline[pos++]= *p1;
  }

  newline[pos] = '\0';
  return(newline);
}
/* ----------------------------------------------------------------------
 * dumplist:
 *  Outputs the current location counter, bytebuf[] array, and
 *  the string 'txt' to the listing file.
 *  This routine is called for every source line encountered in the
 *  source file. (Only in pass 2, and if listing is turned on).
 *
 */
void dumplist(char *txt, int show)
{
  int i,
      j;

  if(show)
    fprintf(listing, "%06X: ", lc);
  else
    fprintf(listing,"        ");

#ifdef _DEBUG
if(bytecount > sizeof(bytebuf))
  fprintf(listing, "\n!@#$ MEMORY SCROG: ByteCount: %d\n", bytecount);
#endif

  j = 0;

  for(i = 0; i < bytecount; i++) 
  {
    fprintf(listing, "%02X ", bytebuf[i]);

    if((++j >= 4) && (bytecount > 4))
    {
      j = 0;
      fprintf(listing,"\n        ");
    }
  }

  while(++j <= 4)
    fprintf(listing,"   ");

  fprintf(listing, " %s\n", padline(txt));
  ResetLineBuffer();
}
/* ----------------------------------------------------------------------
 * DumpSymbols:
 *  Outputs the current location counter, bytebuf[] array, and
 *  the string 'txt' to the listing file.
 *  This routine is called for every source line encountered in the
 *  source file. (Only in pass 2, and if listing is turned on).
 *
 */
void DumpSymbols(void)
{
  SymbolDump(listing);
}
/* ----------------------------------------------------------------------
 * gen* routines:
 *  Place information into the bytebuf[] array, and also
 *  call emitbyte with the byte.
 *
 */
void genbyte(int b)
{
  if(bytecount < sizeof(bytebuf))
    bytebuf[bytecount++] = b;
  
  emitbyte(b);
}
/**********************************************************************/
void genstr(char *s)
{
  while(*s)
    genbyte(*s++);
}
/**********************************************************************/
void  genword(unsigned long w)
{
  genbyte((w & 0xff00) >> 8);
  genbyte((w & 0x00ff));
}
/**********************************************************************
 * generates a 24 bit address
 */
void  gentriple(unsigned long t)
{
  genbyte((t & 0xff0000) >> 16);
  genbyte((t & 0x00ff00) >> 8);
  genbyte((t & 0x0000ff));
}
/**********************************************************************/

