
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.comm;



/**
 * <p>This class is used when data is requested for input but
 * a port is not currently available. This class is used when the
 * <code>java.lang.System</code> class initializes.  
 * <code>System.in</code> is initialized to a <code>NullInputStream</code>
 * so that system initialization routines do not have to open
 * any devices.</p>
 *
 * <p>This class is also useful in other instances.  In Slush, if a 
 * process is run in the background and the input streams are not redirected
 * from a file or some other source, the background process should not be allowed
 * to share the standard input stream <code>System.in</code>.  In this case,
 * the background process is created with a <code>NullInputStream</code> as its
 * standard input.</p>
 *
 * <p>This class duplicates the behavior of a closed stream.</p>
 *
 * @see NullOutputStream
 */
public class NullInputStream
   extends java.io.InputStream
{

   /**
    * Reads a byte of data from this <code>NullInputStream</code>.  The value
    * <code>-1</code> is returned, indicating the stream is closed
    * and no more data will be available. 
    *
    * @return <code>-1</code>, indicating this stream has no more
    * input data
    */
   public int read ()
   {
      return -1;
   }
}
