
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.comm;

import javax.comm.*;

/**
  * This class, part of the loadable device driver interface, is used by
  * <code>javax.comm</code> and <B><I>should not be used by application-level
  * programs.</B></I>
  */
public class TINICommDriver
   implements CommDriver
{

   /** initialize() will be called by the CommPortIdentifier's static
     * initializer. The responsibility of this method is:<br>
     * 1) Ensure that that the hardware is present.<br>
     * 2) Load any required native libraries.<br>
     * 3) Register the port names with the CommPortIdentifier.
     */
   public void initialize ()
   {
      try
      {

         String[] tiniSerialPortNames = TINISerialPort.getPortNames();

         for (int i = 0; i < tiniSerialPortNames.length; i++)
         {

            CommPortIdentifier.addPortName(
               tiniSerialPortNames [i],
               javax.comm.CommPortIdentifier.PORT_SERIAL, this);

         }
      }
      catch (Throwable t){}
   }

   /**
    * <code>getCommPort()</code> will be called by <code>CommPortIdentifier</code>
    * from its <code>open()</code> method. <code>portName</code> is a string that was
    * registered earlier using the <code>CommPortIdentifier.addPortName()</code>
    * method. <code>getCommPort()</code> returns an object that extends either
    * SerialPort or ParallelPort.
    *
    * @param name a port name
    * @param type a port type
    *
    * @return a CommPort object
    */
   public javax.comm.CommPort getCommPort (String name, int type)
   {

      switch (type)
      {

         case javax.comm.CommPortIdentifier.PORT_SERIAL :
            if (TINISerialPort.isOwned(name))
            {

               return null;
            }

            return ( javax.comm.CommPort ) (new com.dalsemi.comm.TINISerialPort(
               name));

         default :
            throw new IllegalArgumentException("port type " + type
                                               + " is not supported");
      }
   }
}
