
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.io;



import com.dalsemi.system.ArrayUtils;
import java.io.CharConversionException;


/**
 *
 */
public class ByteToCharUTF8
   extends ByteToCharConverter
{

   //This buffer needs to be protected from multiple threads
   //trying to convert at once. ->  The InputStreamReader takes
   //care of this
   private byte[]  buffer        = new byte [2];
   private int     count         = 0;
   private boolean errorDetected = false;
	
	int b1;
	int b2;
	boolean error = false;


   /**
    *
    */
   public int getMaxCharCount (byte[] forThis, int start, int end)
   {
 
      //return end - start;
      //this function needs to return how many unicode characters there are here
      //what we do is parse it, every byte that starts with :
      //    0xxxxxxx, 110xxxxx, 1110xxxx
      //is the mark of a new unicode character, and we need to make sure that
      //the end doesn't chop the middle of a character
      int count = 0;

      for (int i = start; i < end; i++)
      {
         if ((forThis [i] & 0x80) == 0x00)        //if this is a 0xxxxxxx byte
            count++;
         else if ((forThis [i] & 0xe0) == 0xc0)   //if this is a 110xxxx byte
         {
            i++;

            if (i < end)
               count++;
         }
         else if ((forThis [i] & 0xf0) == 0xe0)   //if this is a 1110xxxx byte
         {
            i++;
            i++;

            if (i < end)
               count++;
         }
      }

      return count;
   }


   

   /* KLA...new and improved, this time you can maybe understand the code!
    *
    */
   public int convert (byte[] src, int srcStart, int srcEnd, char[] dst,
                       int dstStart, int dstEnd)

	{
		//if (error) System.out.println("GOT AN ERROR!!!");
		char ch = 0;
		int retVal = 0;
		while ((srcStart < srcEnd) && (dstStart < dstEnd))
		{
			//if (error) return 0;

			int b = (src[srcStart++] & 0x0ff);
			switch (count)
			{
				case 0 :if (b >= 0x80)
					{
						count = 1;
						b1 = b;
					}
					else
						ch = (char)b;
					break; 
				case 1 :if ((b1 & 0xf0)==0xe0)
					{
						count = 2;
						b2 = b;
					}
					else if (((b1 & 0xe0)==0xc0) && ((b & 0xc0)==0x80))
					{
						count = 0;
						ch = (char) (((b1 & 0x1f) << 6) | (b & 0x3f));
					}
					else
					{
						error = true;
						ch = 0x3f;
						count = 0;
					}
					break; 
				case 2 :if (((b1 & 0xf0)==0xe0) && ((b2 & 0xc0)==0x80) && ((b & 0xc0)==0x80))
					{
						ch = (char) ( ((b1 & 0x0f) << 12 ) | ((b2 & 0x3f) << 6 ) | (b & 0x03f));
						count = 0;
					}
					else
					{
						error = true;
						ch = 0x3f;
						count = 0;
					}
					break;
			}
			if (count == 0)
			{
				dst[dstStart++] = ch;
				retVal++;
			}
		}
		return retVal;
	}
   
   
   /**
    *
    */
   public int flush (char[] buff, int start, int length)
      throws CharConversionException
   {
   //   count         = 0;
      errorDetected = false;

      if (length == 0)
         return 0;

      throw new CharConversionException("UTF8 Cannot convert more data");
   }

   /**
    *
    */
   public String getName ()
   {
      return "UTF8";
   }
}
