
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
package com.dalsemi.onewire.debug;

/**
 * <p>This class is intended to help both developers of the 1-Wire API for 
 * Java and developers using the 1-Wire API for Java to have a standard
 * method for printing debug messages.  Applications that want to see debug messages
 * should call  the <code>setDebugMode(boolean)</code> method.
 * Classes that want to print information under debugging 
 * circumstances should call the <code>debug(String)</code>
 * method.</p>
 *
 * <p>Debug printing is turned off by default.</p>
 *
 * @version    0.00, 20 Mar 2001
 * @author     K
 */
public class Debug
{
	private static boolean DEBUG = false;

   /**
    * Sets the debug printing mode for this application.
    *
    * @param <code>true</code> to see debug messages, <code>false</code>
    *        to suppress them
    */
	public static final void setDebugMode(boolean onoff)
	{
		DEBUG = onoff;
	}

   /**
    * Gets the debug printing mode for this application.
    * 
    * @return <code>true</code> indicates debug messages are on,
    *         <code>false</code> supresses them.
    */
	public static final boolean getDebugMode()
	{
		return DEBUG;
	}
   
   /**
    * Prints the specified <code>java.lang.String</code> object 
    * if debug mode is enabled.  This method calls <code>System.out.println(String)</code>,
    * and pre-pends the <code>String</code> ">> " to the message, so 
    * taht if a program were to call (when debug mode was enabled):
    * <code><pre>
    *     com.dalsemi.onewire.debug.Debug.debug("Some notification...");
    * </pre></code>
    * the resulting output would look like:
    * <code><pre>
    *     >> Some notification...
    * </pre></code>
    *
    * @param the message to print out if in debug mode
    */
	public static final void debug(String x)
	{
		if (DEBUG) 
		    System.out.println(">> "+x);
	}

}