
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

/*
 * Handler.java
 * A URLStreamHandler for the file protocol.  On the TINI board, assuming
 * that only the local file system is available.
 *
 * @author KLA
 */
package com.dalsemi.protocol.file;



import java.io.IOException;
import java.net.*;
import java.util.StringTokenizer;


public class Handler
   extends URLStreamHandler
{

   //a file url looks like this:
   //      file://<host>/<path>
   //simple enough, right?
   //    String file=null;
   protected URLConnection openConnection (URL u)
      throws IOException
   {
      if (u.getHost() != null)
      {
         if (u.getHost().length() > 0)
         {
            if (!u.getHost().equalsIgnoreCase("localhost"))
               throw new MalformedURLException(
                  "<file://> protocol limited to TINI File System (file://localhost/<file>).");
         }
      }

      return new Connection(u);
   }

   protected void parseURL (URL url, String spec, int start, int limit)
   {

      // Let the superclass do the parsing.
      super.parseURL(url, spec, start, limit);

      // Check for hostname, if it's null, use localhost
      if (url.getHost() == null)
      {
         setURL(url, url.getProtocol(), "localhost", url.getPort(),
                url.getFile(), url.getRef());
      }
   }
}
