
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.protocol.http;



import java.net.*;
import java.io.*;
import java.util.*;
import com.dalsemi.protocol.BasicClient;
import com.dalsemi.protocol.HeaderManager;


public class HTTPClient
   extends BasicClient
{
   //InputStream   reader = null;   //for reading data and command responses
   HeaderManager hm     = null;
   HeaderManager reqHm  = null;   //for request headers
   
   private HTTPOutputStream hout = null;

   /* Create a connection to an FTP server running at the
    * specified HOST on the default FTP port
    */

   /**
    * Constructor HTTPClient
    *
    *
    * @param SERVER
    * @param h
    *
    * @throws IOException
    *
    */
   public HTTPClient (String SERVER, HeaderManager h)
      throws IOException
   {
      super(SERVER, 80);   //well known HTTP port

      hm = h;
   }

   /* Create a connection to an FTP server running at the
    * specified HOST:PORT
    */

   /**
    * Constructor HTTPClient
    *
    *
    * @param SERVER
    * @param PORT
    * @param h
    *
    * @throws IOException
    *
    */
   public HTTPClient (String SERVER, int PORT, HeaderManager h)
      throws IOException
   {
      super(SERVER, PORT);

      hm = h;
   }

   /* Create a connection to an HTTP server running at the
    * specified HOST on the default HTTP port
    */

   /**
    * Constructor HTTPClient
    *
    *
    * @param SERVER
    *
    * @throws IOException
    *
    */
   public HTTPClient (String SERVER)
      throws IOException
   {
      super(SERVER, 80);   //well known HTTP port

      hm = new HeaderManager();
   }

   /* Create a connection to an HTTP server running at the
    * specified HOST:PORT
    */

   /**
    * Constructor HTTPClient
    *
    *
    * @param SERVER
    * @param PORT
    *
    * @throws IOException
    *
    */
   public HTTPClient (String SERVER, int PORT)
      throws IOException
   {
      super(SERVER, PORT);

      hm = new HeaderManager();
   }

   /**
    * Method reConnect
    *
    *
    * @return
    *
    */
   public boolean reConnect ()
   {
      try
      {
         int timeout = control.getSoTimeout();

         cin.close();
         cout.close();
         control.close();

         control = new Socket(controlServer, controlPort);

         control.setSoTimeout(timeout);

         cin  = new BufferedInputStream(control.getInputStream());
         cout = control.getOutputStream();
      }
      catch (IOException ioe)
      {
         return false;
      }

      return true;
   }

   /**
    * Method clearHeaderManagers
    *
    *
    */
   public void clearHeaderManagers ()
   {
      hm = new HeaderManager();
   }

   /**
    * Method getHeaderManager
    *
    *
    * @return
    *
    */
   public HeaderManager getHeaderManager ()
   {
      return hm;
   }

   /**
    * Method getOutputStream
    *
    *
    * @return
    *
    */
   public HTTPOutputStream getOutputStream ()
   {
      if (hout == null)
        hout = new HTTPOutputStream(cout, reqHm);
      return hout;
   }

   /**
    * Method getInputStream
    *
    *
    * @return
    *
    */
   public InputStream getInputStream ()
   {
      return cin;
   }

   /**
    * Method setRequestHeaderManager
    *
    *
    * @param h
    *
    */
   public void setRequestHeaderManager (HeaderManager h)
   {
      reqHm = h;
   }

   /* Sends a command to the HTTP server and returns
    * the stream to the entity body from the HTTP server.  Also
    * stores all the returned headers in the header manager.
    * The java.net class HTTPURLConnection already has methods
    * for getting the response code stuff outta there.
    */

   /**
    * Method issueCommand
    *
    *
    * @param cmd
    *
    * @return
    *
    * @throws IOException
    *
    */
   public InputStream issueCommand (String cmd)
      throws IOException
   {
      InputStream reader = null;
    
      try
      {
         if (cmd != null)
         {
/*            if (!((cmd.startsWith("GET")) || (cmd.startsWith("HEAD"))))
            {
               reader = command(cmd, false);

               return reader;
            }
            else*/
            {
               command(cmd, false);

               if (cmd.startsWith("POST"))
               {
                    reqHm.setIfNotSet("Content-length", "0");
               }

               //need to send the headers
               if (reqHm != null)
               {
                  for (int i = 0; i < reqHm.count(); i++)
                  {
                     command(new String(reqHm.getKey(i) + ": " + reqHm.get(i)
                                        + "\r\n"), false);
                  }
               }

               reader = command("\r\n");
               
               if (hout != null)
               {
                  //then this is a POST request
                  cout.write(hout.baos.toByteArray());
                  //return reader;
               }
            }
         }

         //first check to see if the first 7 bytes are 'HTTP/1.'.  Of so, parse the headers.  If not, skip 'em.
         reader = new BufferedInputStream(reader);
         PushbackInputStream preader = new PushbackInputStream(reader, 7);
         byte[] temp = new byte[7];
         int amount = preader.read(temp);
         preader.unread(temp, 0, amount);

         boolean processHeaders = false;
         if (amount == 7)
         {
             if (com.dalsemi.system.ArrayUtils.arrayComp(temp, 0, "HTTP/1.".getBytes(), 0, 7))
             {
                 processHeaders = true;
             }
         }
         
         if (!processHeaders)
             return preader;


         //read lines one by one and put them into headers until we
         //run into "\r\n\r\n", or the 'empty line'
         String       line = null;
         StringBuffer sb;
         int          r, i, count;
         boolean validread;

         do
         {
            // i toyed with the idea of doing this with a buffered reader,
            // but the problem here is that a buffered reader will 'buffer up'
            // and read too much, which will already have been read from 
            // the input stream, and thus not be available to the application
            // that is recieving this input stream
            
            r  = 0;
            i = -1;
            count = 0;
            sb = new StringBuffer();
            r  = preader.read();

            while (r != '\n')
            {
               if (r == -1)
                  throw new IOException("Bad HTTP Response");
               if ((r == ':') && (i==-1))  //only get the first ':'
                  i = count;
               count++;
               sb.append(( char ) r);

               r = preader.read();
            }

            sb.append(( char ) r);

            line = sb.toString().trim();
            validread = (!line.equals(""));
            if (validread)
            {

               //now break the line into content parts
               //i already has the index of ':'
               //i = line.indexOf(':');

               if (i < 0)
                  hm.add(null, line);
               else
                  hm.add(line.substring(0, i), line.substring(i + 1).trim());
            }
         }
         while (validread); //(!line.equals(""));

         return preader;
      }
      catch (IOException e)
      {

         // Make sure superclass' socket is closed on IO exception.
         control.close();

         throw e;
      }
   }

   /* Outputs meaningful IOExceptions in the event of bad
    * HTTP return codes.
    */

   /**
    * Method analyze
    *
    *
    * @param code
    *
    * @throws IOException
    *
    */
   public void analyze (int code)
      throws IOException
   {
      String temp = null;
      switch (code)
      {

         case 301 :
            temp = "HTTP: 301 Moved Permanently";
            break;
         case 302 :
            temp = "HTTP: 302 Moved Temporarily";
            break;
         case 304 :
            temp = "HTTP: 304 Not Modified";
            break;
         case 400 :
            temp = "HTTP: 400 Bad Request";
            break;
         case 401 :
            temp = "HTTP: 401 Unauthorized";
            break;
         case 403 :
            temp = "HTTP: 403 Forbidden";
            break;
         case 404 :
            temp = "HTTP: 404 Not Found";
            break;
         case 500 :
            temp = "HTTP: 500 Internal Server Error";
            break;
         case 501 :
            temp = "HTTP: 501 Not Implemented";
            break;
         case 502 :
            temp = "HTTP: 502 Bad Gateway";
            break;
         case 503 :
            temp = "HTTP: 503 Service Unavailable";
      }

      if (temp==null)
      {
        if (code > 499)
            temp = "HTTP: Server Failure";
        else if (code > 399)
            temp = "HTTP: Bad Request Syntax/Client Error";
      }
      if (temp!=null)
        throw new IOException(temp);
   }
}
