
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.protocol.mailto;



import java.io.*;
import com.dalsemi.protocol.*;


/**
 * Print values to SmtpClient output stream.
 *
 *
 * @CIM
 */
public class MailToPrintStream
   extends PrintStream
{
   private Connection  mailout;
   private int         lastChar;
   private static byte retArray [];

   /**
    * Constructs new MailToPrintStream to the outputstream and smtpclient
    * defined by the parameters
    *
    *
    * @param outputstream
    * @param smtpclient
    *
    */
   MailToPrintStream (OutputStream outputstream, Connection connection)
   {
      super(outputstream);

      lastChar = '\n';   // Line feed
      mailout  = connection;
   }

   /**
    * Send message and clear input/output streams
    *
    */
   public void close ()
   {

      // end message to server
      // close output and input streams and connection
      if (mailout == null)
         return;

      if (lastChar != '\n')
      {

         // need blank line before ending . to end message
         // and separate line for subject
         write('\r');   // Carriage Return
         write('\n');   // Line feed
      }

      try
      {
         mailout.sendSMTPCommand(".", 250);   // end message
      }
      catch (IOException e)
      {
      }

      try
      {
         mailout.printer      = null;
         mailout.socketReader = null;

         mailout.closeConnection();
      }
      catch (IOException e)
      {
      }
   }

   /**
    * Writes asciiChar to connection output stream
    *
    *
    * @param asciiChar
    *
    */
   public void write (int asciiChar)
   {
      try
      {
         if ((lastChar == '\n') && (asciiChar == '.'))
            out.write('.');
         else
         {
            if (asciiChar == '\n')
            {
               out.write('\r');
            }

            out.write(asciiChar);
         }

         lastChar = asciiChar;

         return;
      }
      catch (IOException e)
      {
         return;
      }
   }

   /**
    * Writes lenght bytes of byteArray at offset i to connection
    * output stream
    *
    *
    * @param byteArray array of bytes to write
    * @param i offset
    * @param length length of byte array
    *
    */
   public void write (byte byteArray [], int i, int length)
   {
      byte[] message = new byte [length];
      int    count   = 0;

      try
      {
         byte nextByte;

         for (int j = 0; j < length; j++)
         {
            nextByte = byteArray [j+i];

            if ((lastChar == '\n') && (nextByte == '.'))
            {
               message [count++] = ( byte ) '.';

               out.write(message, 0, count);

               count = 0;
            }
            else
            {

               // force carriage return
               if (nextByte == '\n')
               {
                  if (lastChar != '\r')
                    message [count++] = ( byte ) '\r';

                  out.write(message, 0, count);

                  count = 0;
               }

               message [count++] = nextByte;
            }

            lastChar = nextByte;
         }

         count -= 1;

         if (count >= 0)
         {
            out.write(message, 0, count + 1);

            count = 0;
         }
      }
      catch (IOException ex)
      {
         return;
      }
   }

   /**
    * Prints String s to connection output stream
    *
    * @param s - string to write
    *
    */
   public void print (String s)
   {
      int length = 0;
      int count  = 0;

      length = s.length();

      byte bArray [] = s.getBytes();

      while (length > 128)
      {
         write(bArray, count, 128);

         count  += 128;
         length -= 128;
      }

      if (length > 0)
         write(bArray, count, length);
   }

   /**
   * Prints String s to connection output stream followed
   * by a line feed and return
   *
   * @param s - string to write
   *
   */
   public void println (String s)
   {
//      if (retArray == null)
//      {
//         retArray = new byte []{ 0x0d, 0x0a };
//      }

      print(s);
//      write(retArray, 0, 2);
      write(0x0a);
   }
}
