/****************************************************************************
*
*  Copyright (C) 1999,2000 Dallas Semiconductor Corporation. 
*  All rights Reserved. Printed in U.S.A.
*  This software is protected by copyright laws of
*  the United States and of foreign countries.
*  This material may also be protected by patent laws of the United States 
*  and of foreign countries.
*  This software is furnished under a license agreement and/or a
*  nondisclosure agreement and may only be used or copied in accordance
*  with the terms of those agreements.
*  The mere transfer of this software does not imply any licenses
*  of trade secrets, proprietary technology, copyrights, patents,
*  trademarks, maskwork rights, or any other form of intellectual
*  property whatsoever. Dallas Semiconductor retains all ownership rights.
*
*     Module Name: PollMemoryCommand.java
*/
package com.dalsemi.slush.command;

import java.io.*;
import java.util.*;
import com.dalsemi.shell.server.*;

/**
 * Sends the free RAM to a given port on a specified interval.
 *
 *
 * @author Lorne Smith
 * @version 1.0
 */
public class PollMemoryCommand implements SlushCommand, MemoryReporterListener
{
    static long interval = 1000;
    static int port = 4401;
    static boolean started = false;
    static MemoryReporter reporter;

    /**
     * Returns a string describing the usage of the command and its arguments if any.
     *
     *
     * @return The usage statement for this command.
     */
    public String getUsageString()
    {
        return "pollmemory [options]\r\n\r\nStarts a thread that will send the free RAM\r\nto the given port every <interval> milliseconds." +
               "\r\n [-i interval] Report interval (ms)." +
               "\r\n [-p port]     Port to use." +
               "\r\n [-s]          Stop the current memory reporter.";
    }

    /**
     * Method called when this command is executed from the command prompt.
     *
     *
     * @param  in   The <code>InputStream</code> this command will use to receive input.
     * @param  out  The <code>PrintStream</code> should use to report any non-critical messages.
     * @param  err  The <code>PrintStream</code> should use to report any critical messages.
     * @param  args The command line arguments passed to this command.
     * @param  env  The current environment variables for this session.
     * @throws Exception
     */
    public void execute(SystemInputStream in, SystemPrintStream out, SystemPrintStream err, 
                        String[] args, Hashtable env) throws Exception
    {
        boolean flag1 = false;
        boolean flag2 = false;
        
        for(int i = 0; i < args.length; i++)
        {
            if(args[i].equals("-i"))
            {
                interval = Long.parseLong(args[++i]);
                flag1 = true;
            }
            else if(args[i].equals("-p"))
            {
                port = Integer.parseInt(args[++i]);
                flag1 = true;
            }
            else if(args[i].equals("-s"))
            {
                flag2 = true;
                if(started)
                {
                    reporter.stopTest();
                }
            }
        }
        if(!flag2 || flag1)
        {
            if(!started)
            {
                if(com.dalsemi.tininet.TININet.getIPAddress().equals(""))
                {
                    out.println("You must have your IP address set before using this command.");
                }
                else
                {
                    out.print("Starting up polling thread...");
                    reporter = new MemoryReporter();
                    reporter.startTest(interval, port, out, this);
                    out.println("[Done]");
                    out.println("Memory being reported to port " + port + " every " + interval + " ms.");
                    started = true;
                }
            }
            else
            {
                out.println("Memory already being reported.");
                out.println("Use 'pollmemory -s' to stop polling.");
            }
        }
    }

    public void reporterStopped()
    {
        started = false;
    }
}