
/*---------------------------------------------------------------------------
 * Copyright (C) 1999,2000 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */

package com.dalsemi.system;

/**
 * This class allows byte wide access to available processor ports.
 *
 */
public class BytePort
{
  /**
    * Port 5, pins 14-21.
    * Using this port will disable CAN 0, I2C pins,
    * serial 1, the external 1-Wire port,
    * and any external chip enables (/PCE0-/PCE3).
    * When using this port be aware of your hardware setup!
    */
  public static final byte Port5 = 0;         // Port 5

  private static final byte FUNC_WRITE    = 0;
  private static final byte FUNC_READ     = 1;

  private byte portName;
  private byte[] bytePortWriteArray = new byte[3];
  private byte[] bytePortReadArray = new byte[3];

  private static native int bytePortNative (int function,byte port,int value);

  /*
  * No one is allowed here.
  */
  private BytePort ()
  {
  }

 /**
  * Creates a new instance of <code>BytePort</code> using the port specified.
  *
  * @param bitname descriptor of port to use (e.g. Port5)
  *
  */
  public BytePort (byte portname)
  {
    portName = portname;
  }

  /**
    * Writes a single byte to the port.
    *
    * @param value single byte to write
    */
   public void write (int value)
   {
     bytePortNative(FUNC_WRITE, portName, value);
   }

   /**
    * Reads a single byte from the port.
    *
    * @return byte read
    */
   public int read ()
   {
     return bytePortNative(FUNC_READ, portName, 0);
   }

}
