/*
 * Copyright (C) 2001 Dallas Semiconductor Corporation.
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 */

package com.dalsemi.tininet;

import java.net.*;

/** This subclass of <CODE>{@link java.net.DatagramSocket}
 * </CODE> is a faster and memory conserving version
 * of <CODE>java.net.DatagramSocket</CODE>.
 * <CODE>TINIDatagramSocket</CODE> is a direct drop-in
 * replacement. The <CODE>receive()</CODE> method is
 * the only method behaving slightly differently. Unlike
 * its superclass, <CODE>TINIDatagramSocket.receive(p)</CODE>
 * does not allocate a new <CODE>InetAddress</CODE> object
 * when the <CODE>DatagramPacket</CODE> p's <CODE>address</CODE>
 * is not null.
 */
public class TINIDatagramSocket extends DatagramSocket
{
  /** Constructs a <CODE>TINIDatagramSocket</CODE> and binds
   * it to any available port on the local host machine.
   * @throws SocketException if the socket could not be opened, or the
   * socket bind operation failed.
   */  
  public TINIDatagramSocket ()
     throws SocketException
  {
    super();
  }

  /** Constructs a <CODE>TINIDatagramSocket</CODE> and binds
   * it to the specified port on the local host machine.
   * @param port local port to use.
   * @throws SocketException if the socket could not be opened, or the
   * socket could not bind to the specified
   * local port.
   */  
  public TINIDatagramSocket(int port)
    throws SocketException
  {
    super(port);
  }

  /** Constructs a <CODE>TINIDatagramSocket</CODE> and binds
   * it to the specified port and the specified address
   * on the local host machine.
   * @param port local port to use.
   * @param laddr local address to use.
   * @throws SocketException if the socket could not be opened, or the
   * socket could not bind to the specified
   * local port and address.
   */  
  public TINIDatagramSocket(int port, InetAddress laddr)
    throws SocketException
  {
    super(port, laddr);
  }
}
